@@ -22,7 +22,7 @@
 
 AccountDialog::AccountDialog(PumpController *pumpController, QWidget *parent) : QWidget(parent)
 {
-    this->setWindowTitle("Dianara - " + tr("Account Configuration"));
+    this->setWindowTitle(tr("Account Configuration") + " - Dianara");
     this->setWindowIcon(QIcon::fromTheme("dialog-password"));
     this->setWindowFlags(Qt::Window);
     this->setWindowModality(Qt::ApplicationModal);
@@ -187,6 +187,9 @@ AccountDialog::~AccountDialog()
 
 void AccountDialog::askForToken()
 {
+    QSettings settings;
+    settings.setValue("isApplicationAuthorized", false); // Until steps are completed
+
     // Don't fail if there're spaces before or after the ID, they're only human ;)
     userIDLineEdit->setText(userIDLineEdit->text().trimmed());
 
