@@ -119,8 +119,9 @@ ASObject::ASObject(QVariantMap objectMap,
             qDebug() << "No proxyed link to general file, fetching regular link";
             fileUrl = objectMap.value("fileUrl").toString();
         }
-        // FIXME: get mimeType
-        qDebug() << "file mimeType:" << objectMap.value("mimeType").toString();
+
+        mimeType = objectMap.value("mimeType").toString();
+        qDebug() << "file mimeType:" << mimeType;
     }
 
 
@@ -243,6 +244,11 @@ QString ASObject::getFileUrl()
     return this->fileUrl;
 }
 
+QString ASObject::getMimeType()
+{
+    return this->mimeType;
+}
+
 QString ASObject::getLikesCount()
 {
     return this->likesCount;
