@@ -56,6 +56,7 @@ public:
     QString getAudioUrl();
     QString getVideoUrl();
     QString getFileUrl();
+    QString getMimeType();
 
     QString getLikesCount();
     QString getCommentsCount();
@@ -97,6 +98,7 @@ private:
     QString audioUrl;
     QString videoUrl;
     QString fileUrl;
+    QString mimeType;
 
     QString likesCount;
     QString commentsCount;
