@@ -127,8 +127,43 @@ AudienceSelector::~AudienceSelector()
  */
 void AudienceSelector::resetLists()
 {
-    this->selectedListWidget->clear();
     this->peopleWidget->resetWidget();
+
+    this->selectedListWidget->clear();
+    restoreSelected();
+}
+
+void AudienceSelector::deletePrevious()
+{
+    foreach (QListWidgetItem *item, previousItems)
+    {
+        delete item;
+    }
+    previousItems.clear();
+}
+
+
+void AudienceSelector::saveSelected()
+{
+    qDebug() << "AudienceSelector::saveSelected()";
+    // Clear and delete all first
+    this->deletePrevious();
+
+    int totalItems = this->selectedListWidget->count();
+    for (int counter = 0; counter < totalItems; ++counter)
+    {
+        this->previousItems.append(selectedListWidget->item(counter)->clone());
+    }
+}
+
+
+void AudienceSelector::restoreSelected()
+{
+    qDebug() << "AudienceSelector::restoreSelected()";
+    foreach (QListWidgetItem *item, this->previousItems)
+    {
+        this->selectedListWidget->addItem(item->clone());
+    }
 }
 
 
@@ -176,6 +211,8 @@ void AudienceSelector::setAudience()
 
     emit audienceSelected(selectorType, addressList);
 
+    saveSelected();  // To restore the list later, if the dialog is shown again
+
     this->hide();  // Don't close(), because that resets the lists =)
 }
 
