@@ -406,7 +406,7 @@ void Comment::quoteComment()
 void Comment::editComment()
 {
     emit commentEditRequested(this->commentId,
-                              this->contentLabel->text());
+                              this->commentOriginalText);
 }
 
 
