@@ -581,7 +581,7 @@ void Composer::cancelPost()
 
     if (this->document()->isEmpty())
     {
-        cancelConfirmed = 0; // Cancelling doesn't need confirmation if it's empty
+        cancelConfirmed = 0; // Canceling doesn't need confirmation if it's empty
     }
     else
     {
