@@ -34,6 +34,8 @@
 #include <QPushButton>
 #include <QSettings>
 #include <QAction>
+#include <QFileDialog>
+#include <QMessageBox>
 
 #include <QDebug>
 
@@ -65,6 +67,7 @@ signals:
 
 public slots:
     void saveConfiguration();
+    void pickCustomIconFile();
 
 
 private:
@@ -106,6 +109,15 @@ private:
     QCheckBox *notifyHLMWCheckbox;
 
 
+    // Tab 4, system tray icon options
+    QWidget *systrayOptionsWidget;
+    QFormLayout *systrayOptionsLayout;
+
+    QComboBox *systrayIconTypeCombobox;
+    QPushButton *systrayCustomIconButton;
+
+    QString systrayCustomIconFN;
+
 
     // Widgets below the tab widget
     QLabel *dataDirectoryLabel;
