@@ -71,9 +71,12 @@ ListsManager::ListsManager(PumpController *pumpController,
 
     this->newListNameLineEdit = new QLineEdit();
     newListNameLineEdit->setPlaceholderText(tr("Type a name for the new list..."));
+    connect(newListNameLineEdit, SIGNAL(textChanged(QString)),
+            this, SLOT(enableDisableCreateButton(QString)));
     connect(newListNameLineEdit, SIGNAL(returnPressed()),
             this, SLOT(createList()));
 
+
     this->newListDescTextEdit = new QTextEdit();
     newListDescTextEdit->setToolTip(tr("Type an optional description here"));
     newListDescTextEdit->setTabChangesFocus(true);
@@ -81,6 +84,7 @@ ListsManager::ListsManager(PumpController *pumpController,
 
     this->createListButton = new QPushButton(QIcon::fromTheme("edit-table-insert-row-above"),
                                              tr("Create L&ist"));
+    createListButton->setDisabled(true); // Disabled until user types a name
     connect(createListButton, SIGNAL(clicked()),
             this, SLOT(createList()));
 
@@ -310,6 +314,24 @@ void ListsManager::deleteList()
     }
 }
 
+
+/*
+ * Enable or disable the "Create List" button
+ *
+ */
+void ListsManager::enableDisableCreateButton(QString listName)
+{
+    if (listName.trimmed().isEmpty())
+    {
+        this->createListButton->setDisabled(true);
+    }
+    else
+    {
+        this->createListButton->setEnabled(true);
+    }
+}
+
+
 /*
  * Enable or disable the "remove person" and "delete list" buttons
  * according to what's currently selected
