@@ -56,15 +56,14 @@ int main(int argc, char *argv[])
 {
     QApplication dianaraApp(argc, argv);
     dianaraApp.setApplicationName("Dianara");
-    dianaraApp.setApplicationVersion("1.2.0");
+    dianaraApp.setApplicationVersion("1.2.1");
     dianaraApp.setOrganizationName("JanCoding");
     dianaraApp.setOrganizationDomain("jancoding.wordpress.com");
 
-    std::cout << QString("%1 v%2 - JanKusanagi 2012-2014\n\n")
-                     .arg(dianaraApp.applicationName())
-                     .arg(dianaraApp.applicationVersion()).toStdString();
-    std::cout << QString("Built with Qt v%1\n").arg(qVersion()).toStdString();
-    std::cout << QString("Running with Qt v%1\n\n").arg(QT_VERSION_STR).toStdString();
+    std::cout << QString("Dianara v%1 - JanKusanagi 2012-2014\n")
+                        .arg(dianaraApp.applicationVersion()).toStdString();
+    std::cout << QString("- Built with Qt v%1\n").arg(qVersion()).toStdString();
+    std::cout << QString("- Running with Qt v%1\n\n").arg(QT_VERSION_STR).toStdString();
     std::cout.flush();
 
     // To make the mswin version of Dianara distributable as a standalone package
@@ -72,18 +71,68 @@ int main(int argc, char *argv[])
     dianaraApp.addLibraryPath("./plugins/");
 #endif
 
-    // Register custom message handler, to hide debug messages unless specified
-    if (qApp->arguments().contains("--debug", Qt::CaseInsensitive))
+
+    QStringList cmdLine = qApp->arguments();
+    cmdLine.removeFirst();
+
+    bool debugMode = false;
+    bool nextParameterIsConfig = false;
+
+    // Parse command line parameters
+    if (!cmdLine.isEmpty())
     {
-        qDebug() << "Debug messages enabled";
+        foreach (QString argument, cmdLine)
+        {
+            // Help
+            if (argument.startsWith("--help", Qt::CaseInsensitive))
+            {
+                std::cout << "\nHelp:\n";
+                std::cout << "    " << argv[0] << " [options]\n\n";
+                std::cout << "Options:\n";
+                std::cout << "    --debug           Show debug messages in the terminal\n";
+                std::cout << "    --config [name]   Use a different configuration file\n";
+                std::cout << "\n";
+
+                return 0; // Exit to shell
+            }
+
+            // Use different config file, by setting a different applicationName
+            if (argument.startsWith("--config", Qt::CaseInsensitive))
+            {
+                nextParameterIsConfig = true;
+                cmdLine.removeAll(argument);
+            }
+            else if (nextParameterIsConfig)
+            {
+                QString configName = "Dianara_" + argument.toLower();
+                dianaraApp.setApplicationName(configName);
+                nextParameterIsConfig = false;
+
+                std::cout << "Using alternate config file: "
+                          << configName.toStdString() << "\n";
+            }
+
+            // Debug mode
+            if (argument.startsWith("--debug", Qt::CaseInsensitive))
+            {
+                debugMode = true;
+                cmdLine.removeAll(argument);
+
+                std::cout << "Debug messages enabled\n";
+            }
+        }
     }
-    else
+
+
+    // Register custom message handler, to hide debug messages unless specified
+    if (!debugMode)
     {
     #ifdef Q_OS_WIN
         FreeConsole();
     #endif
 
-        qDebug() << "To see debug messages while running, use --debug";
+        std::cout << "To see debug messages while running, use --debug\n";
+
     #if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
         qInstallMsgHandler(customMessageHandlerQt4);
     #else
@@ -103,11 +152,14 @@ int main(int argc, char *argv[])
     QString languageFile;
     languageFile = QString(":/translations/dianara_%1")
                    .arg(languageString);
-    qDebug() << "Trying to load translation file:" << languageFile;
+    std::cout << "Using translation file: "
+              << languageFile.toStdString() << "\n";
 
     translator.load(languageFile);
     dianaraApp.installTranslator(&translator);
 
+    std::cout.flush();
+
 
     MainWindow dianaraWindow;
     dianaraWindow.toggleMainWindow();  // show()
