@@ -35,6 +35,7 @@ MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
 
     reallyQuitProgram = false;
     trayIconAvailable = false;
+    trayCurrentCount = 0;
 
 
     QString currentIconset = QIcon::themeName();
@@ -471,25 +472,28 @@ MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
             this, SLOT(setTransientStatusMessage(QString)));
 
 
-    // Add menus
+    // Add menus and toolbar
     createMenus();
+    createToolbar();
+
+    // StatusBar stuff
+    this->createStatusbarWidgets();
+    this->statusBar()->showMessage(tr("Initializing..."));
 
     // Add the system tray icon
     createTrayIcon();
 
 
-    // tmp statusBar stuff
-    this->statusBar()->showMessage(tr("Initializing..."));
-
-
     settings.beginGroup("MainWindow");
 
     // Now set the "view side panel" checkable menu to its saved state
-    // That will also trigger the action to hide/show it
+    // This will also trigger the action to hide/show it
     viewSidePanel->setChecked(settings.value("viewSidePanel", true).toBool());
 
-    // Set the "view status bar" checkable menu to its saved state, which
-    // will also trigger the action to hide/show it
+    // Check view > toolbar if needed; HIDDEN by default
+    viewToolbar->setChecked(settings.value("viewToolbar", false).toBool());
+
+    // Set the "view status bar" checkable menu to its saved state
     viewStatusBar->setChecked(settings.value("viewStatusBar", true).toBool());
 
     settings.endGroup();
@@ -619,7 +623,7 @@ void MainWindow::createMenus()
     sessionMenu = new QMenu(tr("&Session"));
 
     sessionUpdateMainTimeline = new QAction(QIcon::fromTheme("view-refresh"),
-                                            tr("&Update Main Timeline"),
+                                            tr("&Update Timeline"),
                                             this);
     sessionUpdateMainTimeline->setShortcut(QKeySequence(Qt::Key_F5));
     sessionUpdateMainTimeline->setDisabled(true); // Disabled until authorization checked
@@ -628,7 +632,7 @@ void MainWindow::createMenus()
     sessionMenu->addAction(sessionUpdateMainTimeline);
 
     sessionUpdateDirectTimeline = new QAction(QIcon::fromTheme("view-refresh"),
-                                              tr("Update &Messages Timeline"),
+                                              tr("Update &Messages"),
                                               this);
     sessionUpdateDirectTimeline->setShortcut(QKeySequence(Qt::Key_F6));
     sessionUpdateDirectTimeline->setDisabled(true);
@@ -637,7 +641,7 @@ void MainWindow::createMenus()
     sessionMenu->addAction(sessionUpdateDirectTimeline);
 
     sessionUpdateActivityTimeline = new QAction(QIcon::fromTheme("view-refresh"),
-                                                tr("Update &Activity Timeline"),
+                                                tr("Update &Activity"),
                                                 this);
     sessionUpdateActivityTimeline->setShortcut(QKeySequence(Qt::Key_F7));
     sessionUpdateActivityTimeline->setDisabled(true);
@@ -647,7 +651,7 @@ void MainWindow::createMenus()
 
 
     sessionUpdateFavoritesTimeline = new QAction(QIcon::fromTheme("view-refresh"),
-                                                 tr("Update Favorites &Timeline"),
+                                                 tr("Update Fa&vorites"),
                                                  this);
     sessionUpdateFavoritesTimeline->setShortcut(QKeySequence(Qt::Key_F8));
     sessionUpdateFavoritesTimeline->setDisabled(true);
@@ -722,6 +726,16 @@ void MainWindow::createMenus()
 
     viewMenu->addAction(viewSidePanel);
 
+
+    viewToolbar = new QAction(QIcon::fromTheme("configure-toolbars"),
+                                tr("&Toolbar"), this);
+    connect(viewToolbar, SIGNAL(toggled(bool)),
+            this, SLOT(toggleToolbar(bool)));
+    viewToolbar->setCheckable(true);
+
+    viewMenu->addAction(viewToolbar);
+
+
     viewStatusBar = new QAction(QIcon::fromTheme("configure-toolbars"),
                                 tr("Status &Bar"), this);
     connect(viewStatusBar, SIGNAL(toggled(bool)),
@@ -731,6 +745,8 @@ void MainWindow::createMenus()
 
     viewMenu->addAction(viewStatusBar);
 
+    viewMenu->addSeparator();
+
 
     viewFullscreenAction = new QAction(QIcon::fromTheme("view-fullscreen"),
                                        tr("Full &Screen"), this);
@@ -742,7 +758,8 @@ void MainWindow::createMenus()
 
     viewMenu->addAction(viewFullscreenAction);
 
-    viewLogAction = new QAction(QIcon::fromTheme("text-x-log"),
+    viewLogAction = new QAction(QIcon::fromTheme("text-x-log",
+                                                 QIcon(":/images/log.png")),
                                 tr("&Log"), this);
     connect(viewLogAction, SIGNAL(triggered()),
             logViewer, SLOT(show()));
@@ -818,6 +835,8 @@ void MainWindow::createMenus()
     this->menuBar()->addMenu(helpMenu);
 
 
+    this->menuBar()->setContextMenuPolicy(Qt::PreventContextMenu);
+
 
     // Context menu for the tray icon
     trayShowWindowAction = new QAction(QIcon(":/icon/64x64/dianara.png"),
@@ -847,6 +866,38 @@ void MainWindow::createMenus()
 }
 
 
+void MainWindow::createToolbar()
+{
+    this->mainToolBar = addToolBar(tr("Toolbar"));
+    mainToolBar->setToolButtonStyle(Qt::ToolButtonFollowStyle);
+    mainToolBar->setContextMenuPolicy(Qt::PreventContextMenu);
+    mainToolBar->setMovable(false);
+
+    mainToolBar->addAction(this->sessionUpdateMainTimeline);
+    mainToolBar->addAction(this->sessionUpdateMinorFeed);
+    mainToolBar->addAction(this->sessionMarkAllAsRead);
+    mainToolBar->addSeparator();
+    mainToolBar->addAction(this->settingsFilters);
+    mainToolBar->addAction(this->settingsConfigure);
+
+
+    mainToolBar->hide();
+}
+
+
+void MainWindow::createStatusbarWidgets()
+{
+    this->statusLogButton = new QToolButton();
+    statusLogButton->setIcon(QIcon::fromTheme("text-x-log",
+                                              QIcon(":/images/log.png")));
+    statusLogButton->setToolTip(tr("Open the log viewer"));
+    connect(statusLogButton, SIGNAL(clicked()),
+            viewLogAction, SLOT(trigger()));
+
+    this->statusBar()->addPermanentWidget(statusLogButton);
+    this->statusBar()->setSizeGripEnabled(false);
+}
+
 
 /*
  * Create an icon in the system tray, define its contextual menu, etc.
@@ -895,10 +946,38 @@ void MainWindow::createTrayIcon()
  */
 void MainWindow::setTrayIconPixmap(int count)
 {
-    QPixmap iconPixmap = QIcon::fromTheme("dianara",
-                                          QIcon(":/icon/32x32/dianara.png"))
-                         .pixmap(32, 32)
-                         .scaled(32, 32);
+    this->trayCurrentCount = count;
+
+    QPixmap iconPixmap;
+
+    switch (this->trayIconType)
+    {
+    case 0: // Default
+        iconPixmap = QIcon(":/icon/32x32/dianara.png")
+                     .pixmap(32, 32)
+                     .scaled(32, 32);
+        break;
+
+    case 1: // system iconset, if available
+        iconPixmap = QIcon::fromTheme("dianara",
+                                       QIcon(":/icon/32x32/dianara.png"))
+                                       .pixmap(32, 32)
+                                       .scaled(32, 32);
+        break;
+
+    case 2: // Use your own avatar
+        iconPixmap = this->avatarIconButton->icon()
+                     .pixmap(32, 32)
+                     .scaled(32, 32);
+        break;
+
+    case 3: // Custom icon
+        iconPixmap = this->trayCustomPixmap;
+
+        break;
+
+    }
+
 
     // Paint the number of unread messages on top of the pixmap, if != 0
     if (count > 0)
@@ -907,30 +986,20 @@ void MainWindow::setTrayIconPixmap(int count)
         font.setPixelSize(18);
         font.setBold(true);
 
+        // Draw a pseudo-shadow on the side first
         QPainter painter(&iconPixmap);
+        painter.drawPixmap(0, 0,
+                           32, 32,
+                           QPixmap(":/images/tray-bg.png"));
+
         QPen pen;
-
-        // Paint first, as a sort of shadow
         font.setWeight(QFont::Black);
-        pen.setBrush(Qt::black);
-        painter.setFont(font);
-        painter.setPen(pen);
-        painter.drawText(0, 0,
-                         30, 33,
-                         Qt::AlignRight | Qt::AlignBottom,
-                         QString("%1").arg(count));
-        painter.drawText(0, 0,
-                         32, 35,
-                         Qt::AlignRight | Qt::AlignBottom,
-                         QString("%1").arg(count));
-
-        // Again, the real text
-        font.setWeight(QFont::Bold);
         pen.setBrush(Qt::white);
         painter.setFont(font);
         painter.setPen(pen);
+        // Draw the number of new messages
         painter.drawText(0, 0,
-                         31, 34, // Between the two shadows
+                         32, 32,
                          Qt::AlignRight | Qt::AlignBottom,
                          QString("%1").arg(count));
     }
@@ -996,12 +1065,13 @@ void MainWindow::saveSettings()
     // General main window status
     settings.beginGroup("MainWindow");
 
-    settings.setValue("windowSize", this->size());
-    settings.setValue("windowPosition", this->pos());
+    settings.setValue("windowSize",        this->size());
+    settings.setValue("windowPosition",    this->pos());
     settings.setValue("mainSplitterState", this->mainSplitter->saveState());
 
-    settings.setValue("viewSidePanel", this->viewSidePanel->isChecked());
-    settings.setValue("viewStatusBar", this->viewStatusBar->isChecked());
+    settings.setValue("viewSidePanel",     this->viewSidePanel->isChecked());
+    settings.setValue("viewToolbar",       this->viewToolbar->isChecked());
+    settings.setValue("viewStatusBar",     this->viewStatusBar->isChecked());
 
     settings.endGroup();
 
@@ -1089,6 +1159,19 @@ void MainWindow::updateConfigSettings()
     this->activityTimeline->updateHighlightColors(highlightColorsList);
     this->favoritesTimeline->updateHighlightColors(highlightColorsList);
 
+    // System tray icon type and custom icon filename, if any
+    this->trayIconType = settings.value("systrayIconType", 0).toInt();
+    QString trayIconFilename = settings.value("systrayCustomIconFN").toString();
+    this->trayCustomPixmap = QPixmap(trayIconFilename).scaled(32, 32);
+    if (trayCustomPixmap.isNull()) // Custom icon image is gone or something
+    {
+        trayCustomPixmap = QPixmap(":/icon/32x32/dianara.png");
+    }
+    if (trayIconAvailable)
+    {
+        this->setTrayIconPixmap(trayCurrentCount); // Sync icon
+    }
+
     settings.endGroup();
 
 
@@ -1097,6 +1180,8 @@ void MainWindow::updateConfigSettings()
     qDebug() << "tabsPosition updated:" << tabsPosition << tabWidget->tabPosition();
     qDebug() << "tabsMovable updated:" << tabsMovable;
     qDebug() << "public posts updated:" << publicPosts;
+    qDebug() << "colors updated:" << highlightColorsList;
+    qDebug() << "tray icon type updated:" << trayIconType;
 }
 
 
@@ -1116,7 +1201,7 @@ void MainWindow::toggleWidgetsByAuthorization(bool authorized)
 
     if (authorized)
     {
-        // TODO
+        // TODO FIXME
     }
     else
     {
@@ -1460,8 +1545,12 @@ void MainWindow::setTitleAndTrayInfo(int currentTab)
     currentTabTitle = this->tabWidget->tabText(currentTab);
     currentTabTitle.remove("&"); // Remove accelators
 
-    QString title = currentTabTitle
-                  + " - Dianara";
+    QString title = currentTabTitle;
+    if (!this->userID.isEmpty())
+    {
+        title.append(" - " + userID);
+    }
+    title.append(" - Dianara");
 
     /*  Not sure if I like it
 
@@ -1792,13 +1881,20 @@ void MainWindow::toggleSidePanel(bool shown)
 }
 
 
+void MainWindow::toggleToolbar(bool shown)
+{
+    qDebug() << "Showing toolbar:" << shown;
+    this->mainToolBar->setVisible(shown);
+}
+
+
 /*
  * Hide or show the status bar
  *
  */
 void MainWindow::toggleStatusBar(bool shown)
 {
-    qDebug() << "Showing side panel:" << shown;
+    qDebug() << "Showing status bar:" << shown;
     this->statusBar()->setVisible(shown);
 }
 
@@ -1860,7 +1956,7 @@ void MainWindow::visitTips()
 void MainWindow::aboutDianara()
 {
     QMessageBox::about(this, tr("About Dianara"),
-                       "<b>Dianara v1.2.0</b><br>"
+                       "<b>Dianara v1.2.1</b><br>"
                        "Copyright 2012-2014  JanKusanagi<br><br>"
                        "<a href=\"http://jancoding.wordpress.com/dianara\">"
                        "http://jancoding.wordpress.com/dianara</a><br><br>"
@@ -1868,8 +1964,9 @@ void MainWindow::aboutDianara()
                        + tr("Dianara is a pump.io social networking client.")
                        + "<br><br>"
 
-                       + tr("With Dianara you can see your timelines, create new posts, "
-                            "upload pictures, interact with posts, manage "
+                       + tr("With Dianara you can see your timelines, "
+                            "create new posts, upload pictures and "
+                            "other media, interact with posts, manage "
                             "your contacts and follow new people.")
                        + "<br><br>"
 
@@ -1878,7 +1975,9 @@ void MainWindow::aboutDianara()
                        + "<br><br>"
 
                        + tr("English translation by JanKusanagi.",
-                            "TRANSLATORS: Change this with your language and name ;)")
+                            "TRANSLATORS: Change this with your language and name. "
+                            "If there was another translator before you, add your "
+                            "name after theirs ;)")
                        + "<br><br>"
 
                        + tr("Dianara is licensed under the GNU GPL license, and "
