@@ -26,6 +26,7 @@
 #include <QApplication>
 #include <QSettings>
 #include <QMenuBar>
+#include <QToolBar>
 #include <QStatusBar>
 #include <QSystemTrayIcon>
 #include <QVBoxLayout>
@@ -78,6 +79,8 @@ public:
 
     void prepareDataDirectory();
     void createMenus();
+    void createToolbar();
+    void createStatusbarWidgets();
     void createTrayIcon();
     void setTrayIconPixmap(int count = 0);
 
@@ -136,6 +139,7 @@ public slots:
 
 
     void toggleSidePanel(bool shown);
+    void toggleToolbar(bool shown);
     void toggleStatusBar(bool shown);
     void toggleFullscreen(bool enabled);
 
@@ -176,6 +180,7 @@ private:
     QAction *sessionQuit;
 
     QAction *viewSidePanel;
+    QAction *viewToolbar;
     QAction *viewStatusBar;
     QAction *viewFullscreenAction;
     QAction *viewLogAction;
@@ -191,6 +196,13 @@ private:
     QAction *helpAbout;
     ////////////////////////////////////// End menus
 
+    // Toolbar
+    QToolBar *mainToolBar;
+
+    // Statusbar widgets
+    QToolButton *statusLogButton;
+
+
     QSplitter *mainSplitter;
 
     QWidget *leftSideWidget;
@@ -215,6 +227,9 @@ private:
 
     QSystemTrayIcon *trayIcon;
     bool trayIconAvailable;
+    int trayIconType;
+    QPixmap trayCustomPixmap;
+    int trayCurrentCount;
 
 
 
