@@ -37,14 +37,14 @@ MinorFeed::MinorFeed(PumpController *pumpController,
     // Separator frame, to mark where new items end
     separatorFrame = new QFrame();
     separatorFrame->setFrameStyle(QFrame::HLine);
-    separatorFrame->setMinimumHeight(24);
+    separatorFrame->setMinimumHeight(28);
 
     // Button to get more items (older stuff)
     getMoreButton = new QPushButton(QIcon::fromTheme("list-add",
                                                      QIcon(":/images/list-add.png")),
-                                    tr("Get More"));
+                                    tr("Older Activities"));
     getMoreButton->setFlat(true);
-    getMoreButton->setToolTip(tr("Get older activities"));
+    getMoreButton->setToolTip(tr("Get previous minor activities"));
     connect(getMoreButton, SIGNAL(clicked()),
             this, SLOT(getMoreActivities()));
 
@@ -183,7 +183,7 @@ void MinorFeed::getMoreActivities()
 
 void MinorFeed::setFeedContents(QVariantList activitiesList)
 {
-    this->getMoreButton->setDisabled(true);
+    this->getMoreButton->hide();
 
     if (feedOffset == 0) // Not asking for more, but a reload
     {
@@ -293,6 +293,7 @@ void MinorFeed::setFeedContents(QVariantList activitiesList)
     emit newItemsReceived(newItemsCount, newHighlightedItemsCount);
     qDebug() << "Meanwhile feed updated; New items:" << newItemsCount;
 
+    this->getMoreButton->show();
     this->getMoreButton->setEnabled(true);
 }
 
