@@ -23,7 +23,6 @@
 
 #include <QFrame>
 #include <QVBoxLayout>
-#include <QLabel>
 #include <QFont>
 #include <QVariantList>
 #include <QPushButton>
@@ -34,6 +33,7 @@
 #include "mischelpers.h"
 #include "minorfeeditem.h"
 #include "asactivity.h"
+#include "filterchecker.h"
 
 
 class MinorFeed : public QFrame
@@ -42,14 +42,19 @@ class MinorFeed : public QFrame
 
 public:
     explicit MinorFeed(PumpController *pumpController,
+                       FilterChecker *filterChecker,
                        QWidget *parent = 0);
     ~MinorFeed();
+
+    void updateHighlightColors(QStringList colorList);
     void clearContents();
     void markAllAsRead();
+    void updateFuzzyTimestamps();
 
 signals:
-    void newItemsCountChanged(int count);
-    
+    void newItemsCountChanged(int itemCount);
+    void newItemsReceived(int itemCount, int highlightedCount);
+
 public slots:
     void updateFeed();
     void getMoreActivities();
@@ -65,12 +70,18 @@ private:
     QList<MinorFeedItem *> itemsInFeed;
 
     int newItemsCount;
+    int newHighlightedItemsCount;
 
     QVBoxLayout *mainLayout;
     QVBoxLayout *itemsLayout;
     QPushButton *getMoreButton;
 
+    QFrame *separatorFrame;
+
+    QStringList highlightColorsList;
+
     PumpController *pController;
+    FilterChecker *fChecker;
 };
 
 #endif // MINORFEED_H
