@@ -111,8 +111,14 @@ QString MiscHelpers::getFileMimeType(QString fileUri)
 
     if (magic_load(magicCookie, NULL) != 0)
     {
-        qDebug() << "magic_load() error!";
-        return QString();
+        qDebug() << "magic_load() error! No system-wide libmagic DB?";
+        // Try loading from a subdirectory instead (in mswindows, maybe osx)
+        if (magic_load(magicCookie, "plugins/magic") != 0)
+        {
+            qDebug() << "magic_load() error; Can't load plugins/magic.mgc";
+            magic_close(magicCookie);
+            return QString();
+        }
     }
 
     const char *magicMimeString;
@@ -232,9 +238,10 @@ QStringList MiscHelpers::htmlWithReplacedImages(QString originalHtml, int postWi
 
             int imageWidth = getImageWidth(cachedImageFilename);
             // if width is bigger than the post, make it smaller to fit
-            if (imageWidth > postWidth - 40)
+            if (imageWidth > postWidth - 32)
             {
-                imageWidth = postWidth - 40; // Some margins, to account for a scrollbar
+                // Some margins, to account for a scrollbar or a tab space before the image
+                imageWidth = postWidth - 32;
             }
 
             newHtml.replace(stringPos,
