@@ -31,6 +31,10 @@ PeopleWidget::PeopleWidget(QString buttonText,
     connect(pController, SIGNAL(contactListReceived(QString,QVariantList,int)),
             this, SLOT(updateAllContactsList(QString,QVariantList,int)));
 
+    connect(pController, SIGNAL(contactFollowed(ASPerson*)),
+            this, SLOT(addContact(ASPerson*)));
+    connect(pController, SIGNAL(contactUnfollowed(ASPerson*)),
+            this, SLOT(removeContact(ASPerson*)));
 
 
     this->searchLabel = new QLabel(tr("&Search:"));
@@ -42,11 +46,20 @@ PeopleWidget::PeopleWidget(QString buttonText,
             this, SLOT(filterList(QString)));
 
 
-    allContactsListWidget = new QListWidget();
-    allContactsListWidget->setDragDropMode(QListView::DragDrop);
-    allContactsListWidget->setDefaultDropAction(Qt::CopyAction);
-    allContactsListWidget->setSortingEnabled(true);
-    connect(allContactsListWidget, SIGNAL(activated(QModelIndex)),
+    itemModel = new QStandardItemModel(this);
+
+    filterModel = new QSortFilterProxyModel(this);
+    filterModel->setFilterCaseSensitivity(Qt::CaseInsensitive);
+    filterModel->setSortCaseSensitivity(Qt::CaseInsensitive);
+    filterModel->setSourceModel(itemModel);
+
+    allContactsListView = new QListView();
+    allContactsListView->setAlternatingRowColors(true);
+    allContactsListView->setEditTriggers(QAbstractItemView::NoEditTriggers);
+    allContactsListView->setDragDropMode(QListView::DragDrop);
+    allContactsListView->setDefaultDropAction(Qt::CopyAction);
+    allContactsListView->setModel(filterModel);
+    connect(allContactsListView, SIGNAL(activated(QModelIndex)),
             this, SLOT(returnClickedContact(QModelIndex)));
 
 
@@ -62,7 +75,7 @@ PeopleWidget::PeopleWidget(QString buttonText,
     mainLayout->addWidget(searchLabel);
     mainLayout->addWidget(searchLineEdit);
     mainLayout->addSpacing(2);
-    mainLayout->addWidget(allContactsListWidget);
+    mainLayout->addWidget(allContactsListView);
     mainLayout->addSpacing(4);
     mainLayout->addWidget(addToButton);
     this->setLayout(mainLayout);
@@ -73,18 +86,18 @@ PeopleWidget::PeopleWidget(QString buttonText,
     {
         //allContactsListWidget->setSelectionMode(QListView::ExtendedSelection);
 
-        // FIXME 1.2: For now, Single selection mode in both cases
-        allContactsListWidget->setSelectionMode(QListView::SingleSelection);
+        // FIXME 1.3: For now, Single selection mode in both cases
+        allContactsListView->setSelectionMode(QListView::SingleSelection);
     }
     else     // StandaloneWidget
     {
-        this->setWindowTitle("Dianara - "
-                             + tr("Add a contact to a list"));
+        this->setWindowTitle(tr("Add a contact to a list")
+                             + " - Dianara");
         this->setWindowIcon(QIcon::fromTheme("system-users"));
         this->setWindowFlags(Qt::Window);
         this->setWindowModality(Qt::WindowModal);
 
-        allContactsListWidget->setSelectionMode(QListView::SingleSelection);
+        allContactsListView->setSelectionMode(QListView::SingleSelection);
 
         cancelButton = new QPushButton(QIcon::fromTheme("dialog-cancel"),
                                        tr("&Cancel"));
@@ -94,8 +107,6 @@ PeopleWidget::PeopleWidget(QString buttonText,
     }
 
 
-    this->filterList(""); // Populate the list without filter, initially
-
     qDebug() << "PeopleWidget created";
 }
 
@@ -108,13 +119,45 @@ PeopleWidget::~PeopleWidget()
 
 void PeopleWidget::resetWidget()
 {
-    this->allContactsListWidget->scrollToTop();
+    this->allContactsListView->scrollToTop();
 
     this->searchLineEdit->clear(); // Might also trigger filterLists()
     this->searchLineEdit->setFocus();
 }
 
 
+QStandardItem *PeopleWidget::createContactItem(ASPerson *contact)
+{
+    QStandardItem *item;
+
+    QString contactName = contact->getName().trimmed();
+    if (contactName.isEmpty())
+    {
+        contactName = contact->getId();
+    }
+    QString singleContactString = contactName
+                                  + "   <" + contact->getId() + ">";
+
+    QString avatarFilename = MiscHelpers::getCachedAvatarFilename(contact->getAvatar());
+    QPixmap avatarPixmap = QPixmap(avatarFilename);
+    if (!avatarPixmap.isNull())
+    {
+        item = new QStandardItem(QIcon(avatarPixmap),
+                                 singleContactString);
+    }
+    else
+    {
+        item = new QStandardItem(QIcon::fromTheme("user-identity",
+                                                  QIcon(":/images/no-avatar.png")),
+                                 singleContactString);
+    }
+
+    item->setToolTip(contact->getTooltipInfo());
+
+    return item;
+}
+
+
 
 //////////////////////////////////////////////////////////////////////////////
 ///////////////////////////////// SLOTS //////////////////////////////////////
@@ -128,10 +171,8 @@ void PeopleWidget::resetWidget()
  */
 void PeopleWidget::filterList(QString searchTerms)
 {
-    allContactsListWidget->clear();
-
-    allContactsListWidget->addItems(allContactsStringList.filter(searchTerms,
-                                                                 Qt::CaseInsensitive));
+    this->filterModel->setFilterFixedString(searchTerms);
+    filterModel->sort(0);
 }
 
 
@@ -152,49 +193,82 @@ void PeopleWidget::updateAllContactsList(QString listType,
 
     if (totalReceivedCount == 200)  // Only on first batch
     {
-        allContactsStringList.clear();
+        this->itemModel->clear();
     }
 
-    QString singleContactString;
-    foreach (QVariant contact, contactsVariantList)
+    foreach (QVariant contactVariant, contactsVariantList)
     {
-        singleContactString = contact.toMap().value("displayName").toString();
-        singleContactString.append("  <");
-        singleContactString.append(contact.toMap().value("id").toString().remove(0,5));
-        singleContactString.append(">");
+        ASPerson *contact = new ASPerson(contactVariant.toMap(), this);
+        QStandardItem *item = this->createContactItem(contact);
+        itemModel->appendRow(item);
 
-        allContactsStringList.append(singleContactString);
+        delete contact;
     }
 
+    filterModel->sort(0); // Sort by first column
+}
 
-    // Populate the list widget, using whatever filter is set
-    this->filterList(searchLineEdit->text());
+
+
+void PeopleWidget::addContact(ASPerson *contact)
+{
+    itemModel->appendRow(this->createContactItem(contact));
+
+    contact->deleteLater();
+}
+
+
+void PeopleWidget::removeContact(ASPerson *contact)
+{
+    foreach (QStandardItem *item, itemModel->findItems("<" + contact->getId() + ">",
+                                                       Qt::MatchEndsWith))
+    {
+        itemModel->removeRow(item->row());
+    }
+
+    contact->deleteLater();
 }
 
 
 
 /*
- * Send current contact string in a SIGNAL
+ * Send current contact icon and string in a SIGNAL
  *
  * Used when selecting a row and clicking the "add" button
  *
  */
 void PeopleWidget::returnContact()
 {
-    QString currentContact;
-    if (this->allContactsListWidget->currentRow() != -1)
+    if (this->allContactsListView->currentIndex().row() != -1)
     {
-        currentContact = allContactsListWidget->currentItem()->data(0).toString();
-    }
+        QStandardItem *item = itemModel->itemFromIndex(filterModel->mapToSource(allContactsListView->currentIndex()));
 
-    emit addButtonPressed(currentContact);
+        QIcon icon;
+        if (item != 0)
+        {
+            icon = item->icon();
+        }
+
+        emit addButtonPressed(icon,
+                              allContactsListView->currentIndex().data().toString());
+    }
 }
 
+
 /*
  * Used when clicking a contact
  *
  */
 void PeopleWidget::returnClickedContact(QModelIndex modelIndex)
 {
-    emit contactSelected(modelIndex.data().toString());
+    QStandardItem *item = itemModel->itemFromIndex(filterModel->mapToSource(modelIndex));
+
+    QIcon icon;
+    if (item != 0) // Valid item
+    {
+        icon = item->icon();
+    }
+
+    emit contactSelected(icon,
+                         modelIndex.data().toString());
 }
