@@ -38,7 +38,7 @@ Post::Post(ASActivity *activity,
 
     leftColumnFrame = new QFrame();
 
-    // Highlighting
+    ///////////////////////////////////////////////// Highlighting
     this->highlightType = NoHighlight;
     QString highlightPostColor;
 
@@ -70,9 +70,9 @@ Post::Post(ASActivity *activity,
             // CSS for horizontal gradient from configured color to transparent
             QString css = QString("QFrame#LeftFrame "
                                   "{ background-color: "
-                                  "qlineargradient(spread:pad, "
-                                  "x1:0, y1:0, x2:1, y2:0, "
-                                  "stop:0 %1, stop:1 rgba(0, 0, 0, 0)); "
+                                  "  qlineargradient(spread:pad, "
+                                  "  x1:0, y1:0, x2:1, y2:0, "
+                                  "  stop:0 %1, stop:1 rgba(0, 0, 0, 0)); "
                                   "}")
                           .arg(highlightPostColor);
 
@@ -85,6 +85,7 @@ Post::Post(ASActivity *activity,
         }
     }
 
+
     leftColumnLayout = new QVBoxLayout();
 
 
@@ -113,7 +114,8 @@ Post::Post(ASActivity *activity,
     }
     else if (postType == "comment")
     {
-        postTypeString = tr("Comment");
+        postTypeString = tr("Comment",
+                            "Noun, as in object type: a comment");
     }
     else
     {
@@ -422,6 +424,9 @@ Post::Post(ASActivity *activity,
     //////////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////// Right column, content
 
+    rightColumnFrame = new QFrame();
+    rightColumnFrame->setObjectName("RightFrame"); // For css changes later
+
     rightColumnLayout = new QVBoxLayout();
     rightColumnLayout->setAlignment(Qt::AlignTop);
 
@@ -466,7 +471,8 @@ Post::Post(ASActivity *activity,
 
     commentButton = new QPushButton(QIcon::fromTheme("mail-reply-sender",
                                                      QIcon(":/images/button-comment.png")),
-                                    tr("Comment"));
+                                    tr("Comment",
+                                       "verb, for the comment button"));
     commentButton->setToolTip(tr("Comment on this post.") + "<br>"
                               + tr("If you select some text, it will be quoted."));
     commentButton->setFlat(true);
@@ -515,7 +521,8 @@ Post::Post(ASActivity *activity,
 
     if (postIsOwn)
     {
-        editButton = new QPushButton(QIcon::fromTheme("document-edit"), tr("Edit"));
+        editButton = new QPushButton(QIcon::fromTheme("document-edit"),
+                                     tr("Edit"));
         editButton->setToolTip(tr("Modify this post"));
         editButton->setFlat(true);
         editButton->setFont(buttonsFont);
@@ -523,7 +530,8 @@ Post::Post(ASActivity *activity,
                 this, SLOT(editPost()));
         buttonsLayout->addWidget(editButton, 0, Qt::AlignRight);
 
-        deleteButton = new QPushButton(QIcon::fromTheme("edit-delete"), tr("Delete"));
+        deleteButton = new QPushButton(QIcon::fromTheme("edit-delete"),
+                                       tr("Delete"));
         deleteButton->setToolTip(tr("Delete this post"));
         deleteButton->setFlat(true);
         deleteButton->setFont(buttonsFont);
@@ -534,6 +542,7 @@ Post::Post(ASActivity *activity,
     }
 
 
+
     /******************************************************************/
 
     this->pendingImagesList.clear(); // Will add the own "post image", and <img>-based ones
@@ -546,10 +555,13 @@ Post::Post(ASActivity *activity,
         pendingImagesList.append(postImageUrl);
     }
 
+
+    QString attachmentUrl;
     // Get URL of post audio file, if it's "audio" type of post
     if (this->postType == "audio")
     {
         postAudioUrl = activity->object()->getAudioUrl();
+        attachmentUrl = postAudioUrl;
         qDebug() << "we got AUDIO:" << postAudioUrl;
     }
 
@@ -557,6 +569,7 @@ Post::Post(ASActivity *activity,
     if (this->postType == "video")
     {
         postVideoUrl = activity->object()->getVideoUrl();
+        attachmentUrl = postVideoUrl;
         qDebug() << "we got VIDEO:" << postVideoUrl;
     }
 
@@ -564,7 +577,9 @@ Post::Post(ASActivity *activity,
     if (this->postType == "file")
     {
         postFileUrl = activity->object()->getFileUrl();
-        qDebug() << "we got FILE:" << postFileUrl;
+        attachmentUrl = postFileUrl;
+        postFileMimeType = activity->object()->getMimeType();
+        qDebug() << "we got FILE:" << postFileUrl << "; type:" << postFileMimeType;
     }
 
 
@@ -579,6 +594,15 @@ Post::Post(ASActivity *activity,
     this->getPendingImages();
 
 
+    // Widget to download the attached media, if any
+    if (!attachmentUrl.isEmpty())
+    {
+        downloadWidget = new DownloadWidget(attachmentUrl,
+                                            this->postType,
+                                            postAuthorId,
+                                            this->pController);
+    }
+
 
     this->resizesCount = 0;
     // Post resizing takes place in resizeEvent()
@@ -598,25 +622,32 @@ Post::Post(ASActivity *activity,
 
     if (!this->postTitle.isEmpty())
     {
-        rightColumnLayout->addSpacing(4); // 4px here, since we removed the general margins
         rightColumnLayout->addWidget(postTitleLabel, 0);
     }
-    rightColumnLayout->addWidget(postText,      3);
-    rightColumnLayout->addLayout(buttonsLayout, 0);
-    rightColumnLayout->addWidget(commenter,     0);
+    rightColumnLayout->addWidget(postText,           3);
+    if (!attachmentUrl.isEmpty())
+    {
+        rightColumnLayout->addWidget(downloadWidget, 0);
+    }
+    rightColumnLayout->addLayout(buttonsLayout,      0);
+    rightColumnLayout->addWidget(commenter,          0);
     rightColumnLayout->addStretch(0);
 
+    rightColumnFrame->setLayout(rightColumnLayout);
 
     // Set the initial likes, comments and shares (4 most recent)
-    this->setLikes(activity->object()->getLastLikesList(), likesCount);
-    this->commenter->setComments(activity->object()->getLastCommentsList(), commentsCount);
-    this->setShares(activity->object()->getLastSharesList(), sharesCount);
+    this->setLikes(activity->object()->getLastLikesList(),
+                   likesCount);
+    this->commenter->setComments(activity->object()->getLastCommentsList(),
+                                 commentsCount);
+    this->setShares(activity->object()->getLastSharesList(),
+                    sharesCount);
 
 
     mainLayout = new QHBoxLayout();
-    mainLayout->setContentsMargins(1, 1, 3, 1); // Minimal margins, except on right side
+    mainLayout->setContentsMargins(0, 0, 0, 0); // Minimal margins
     mainLayout->addWidget(leftColumnFrame,   2);  // stretch 2/12
-    mainLayout->addLayout(rightColumnLayout, 10); // stretch 10/12
+    mainLayout->addWidget(rightColumnFrame, 10); // stretch 10/12
 
     mainLayout->setAlignment(Qt::AlignLeft | Qt::AlignTop);
     this->setLayout(mainLayout);
@@ -683,7 +714,7 @@ void Post::setPostContents()
                             "<img src=\"" + imageCachedFilename + "\" "
                             "width=\"" + QString("%1").arg(imageWidth) + "\" />"
                             "</a>"
-                            "<br><br>";
+                            "<hr><br>";
         }
         else // use placeholder image while it loads...
         {
@@ -691,37 +722,34 @@ void Post::setPostContents()
                             "<div title=\"" + tr("Image is loading...") + "\">"
                             "<img src=\":/images/image-loading.png\"  />"
                             "</div>"
-                            "<br><br>";
+                            "<hr><br>";
         }
     }
 
     // Embedded audio
     if (!this->postAudioUrl.isEmpty())
     {
-        // FIXME, just show a link for now
         postMediaHtml = "<img src=\":/images/attached-audio.png\" /><br>"
-                        "<b>* EMBEDDED AUDIO, <a href=\""
-                        + postAudioUrl + "\">link here</a>.</b><hr><br>";
+                        "<b>" + tr("Attached Audio") + "</b>"
+                        "<hr><br>";
     }
 
 
     // Embedded video
     if (!this->postVideoUrl.isEmpty())
     {
-        // FIXME, just show a link for now
         postMediaHtml = "<img src=\":/images/attached-video.png\" /><br>"
-                        "<b>* EMBEDDED VIDEO, <a href=\""
-                        + postVideoUrl + "\">link here</a>.</b><hr><br>";
+                        "<b>" + tr("Attached Video") + "</b>"
+                        "<hr><br>";
     }
 
 
-    // Embedded file
+    // Attached file
     if (!this->postFileUrl.isEmpty())
     {
-        // FIXME, just show a link for now
         postMediaHtml = "<img src=\":/images/attached-file.png\" /><br>"
-                        "<b>* ATTACHED FILE, <a href=\""
-                        + postFileUrl + "\">link here</a>.</b><hr><br>";
+                        "<b>" + tr("Attached File") + ": " + postFileMimeType
+                        + "</b><hr><br>";
     }
 
 
@@ -1029,24 +1057,34 @@ void Post::setSharesLabel(int resharesCount)
  */
 void Post::setPostUnreadStatus()
 {
+    // CSS for horizontal gradient from configured color to transparent
+    QString css = QString("QFrame#RightFrame "
+                          "{ background-color: "
+                          "  qlineargradient(spread:pad, "
+                          "  x1:0, y1:0, x2:1, y2:0,  "
+                          "  stop:0 rgba(0, 0, 0, 0), "
+                          "  stop:1 palette(highlight)); "
+                          "}");
+
     if (this->postIsUnread)
     {
-        postAuthorNameLabel->setAutoFillBackground(true);
-        postAuthorNameLabel->setForegroundRole(QPalette::HighlightedText);
-        postAuthorNameLabel->setBackgroundRole(QPalette::Highlight);
-
         postCreatedAtLabel->setAutoFillBackground(true);
         postCreatedAtLabel->setForegroundRole(QPalette::HighlightedText);
         postCreatedAtLabel->setBackgroundRole(QPalette::Highlight);
 
-        this->setAutoFillBackground(true);
-        this->setBackgroundRole(QPalette::Mid);
+        if (!css.isEmpty())
+        {
+            rightColumnFrame->setStyleSheet(css);
+        }
+        else
+        {
+            this->setAutoFillBackground(true);
+            this->setBackgroundRole(QPalette::Mid);
+        }
     }
     else
     {
-        postAuthorNameLabel->setAutoFillBackground(false);
-        postAuthorNameLabel->setForegroundRole(QPalette::WindowText);
-        postAuthorNameLabel->setBackgroundRole(QPalette::Window);
+        rightColumnFrame->setStyleSheet("");
 
         postCreatedAtLabel->setAutoFillBackground(false);
         postCreatedAtLabel->setForegroundRole(QPalette::WindowText);
@@ -1431,7 +1469,6 @@ void Post::redrawImages(QString imageUrl)
 
 
 
-
 ////////////////////////////////////////////////////////////////////////////
 //////////////////////////////// PROTECTED /////////////////////////////////
 ////////////////////////////////////////////////////////////////////////////
