@@ -50,6 +50,8 @@
 #include "imageviewer.h"
 #include "asactivity.h"
 #include "avatarbutton.h"
+#include "downloadwidget.h"
+
 
 class Post : public QFrame
 {
@@ -156,6 +158,7 @@ private:
     QVBoxLayout *rightColumnLayout;
 
     QFrame *leftColumnFrame;
+    QFrame *rightColumnFrame;
 
     QString postId;
     QString postType;
@@ -199,11 +202,14 @@ private:
     QPushButton *editButton;
     QPushButton *deleteButton;
 
+    DownloadWidget *downloadWidget;
+
     QString postTitle;
     QString postImageUrl;
     QString postAudioUrl;
     QString postVideoUrl;
     QString postFileUrl;
+    QString postFileMimeType;
     QString postOriginalText;
 
     QString postCreatedAtString;
