@@ -24,7 +24,7 @@
 ProfileEditor::ProfileEditor(PumpController *pumpController,
                              QWidget *parent) : QWidget(parent)
 {
-    this->setWindowTitle("Dianara - " + tr("Profile Editor"));
+    this->setWindowTitle(tr("Profile Editor") + " - Dianara");
     this->setWindowIcon(QIcon::fromTheme("user-properties"));
     this->setWindowFlags(Qt::Window);
     this->setWindowModality(Qt::NonModal);
@@ -65,6 +65,9 @@ ProfileEditor::ProfileEditor(PumpController *pumpController,
     hometownLineEdit = new QLineEdit();
     bioTextEdit = new QTextEdit();
     bioTextEdit->setTabChangesFocus(true);
+    bioTextEdit->setSizePolicy(QSizePolicy::MinimumExpanding,
+                               QSizePolicy::MinimumExpanding);
+
 
 
     saveButton = new QPushButton(QIcon::fromTheme("document-save"),
@@ -84,29 +87,31 @@ ProfileEditor::ProfileEditor(PumpController *pumpController,
     this->addAction(cancelAction);
 
 
+    // Layout
     this->avatarLayout = new QHBoxLayout();
     avatarLayout->addWidget(avatarLabel);
     avatarLayout->addWidget(changeAvatarButton);
 
+    this->topLayout = new QFormLayout();
+    topLayout->setVerticalSpacing(8);
+    topLayout->addRow(tr("Webfinger ID"), webfingerLabel);
+    topLayout->addRow(tr("E-mail"),       emailLabel);
+
+    topLayout->addRow(tr("Avatar"),       avatarLayout);
+
+    topLayout->addRow(tr("Full &Name"),   fullNameLineEdit);
+    topLayout->addRow(tr("&Hometown"),    hometownLineEdit);
+    topLayout->addRow(tr("&Bio"),         bioTextEdit);
 
     this->bottomLayout = new QHBoxLayout();
-    bottomLayout->setAlignment(Qt::AlignRight);
+    bottomLayout->setAlignment(Qt::AlignRight | Qt::AlignBottom);
     bottomLayout->addWidget(saveButton);
     bottomLayout->addWidget(cancelButton);
 
 
-    this->mainLayout = new QFormLayout();
-    mainLayout->setVerticalSpacing(8);
-    mainLayout->addRow(tr("Webfinger ID"), webfingerLabel);
-    mainLayout->addRow(tr("E-mail"),       emailLabel);
-
-    mainLayout->addRow(tr("Avatar"),       avatarLayout);
-    mainLayout->addRow(tr("Full &Name"),   fullNameLineEdit);
-    mainLayout->addRow(tr("&Hometown"),    hometownLineEdit);
-    mainLayout->addRow(tr("&Bio"),         bioTextEdit);
-
-    mainLayout->addRow(bottomLayout);
-
+    this->mainLayout = new QVBoxLayout();
+    mainLayout->addLayout(topLayout);
+    mainLayout->addLayout(bottomLayout);
     this->setLayout(mainLayout);
 
 
@@ -182,7 +187,7 @@ void ProfileEditor::findAvatarFile()
         // and scale the pixmap to something sane before uploading.
         QPixmap avatarPixmap = QPixmap(newAvatarFilename);
 
-        this->newAvatarContentType = MiscHelpers::getImageContentType(newAvatarFilename);
+        this->newAvatarContentType = MiscHelpers::getFileMimeType(newAvatarFilename);
         if (!avatarPixmap.isNull() && !newAvatarContentType.isEmpty())
         {
             this->avatarLabel->setPixmap(QPixmap(newAvatarFilename).scaled(64, 64,
@@ -239,3 +244,15 @@ void ProfileEditor::sendProfileData(QString newImageUrl)
 
     this->hide(); // close() would end the program if main window is hidden
 }
+
+
+/****************************************************************************/
+/****************************** PROTECTED ***********************************/
+/****************************************************************************/
+
+
+void ProfileEditor::closeEvent(QCloseEvent *event)
+{
+    this->hide();
+    event->ignore();
+}
