@@ -49,36 +49,41 @@ Publisher::Publisher(PumpController *pumpController,
             this, SLOT(updateToCcFields(QString,QStringList)));
 
 
+    QString titleTooltip = tr("Brief title for the post. Setting a "
+                              "title helps make the Meanwhile feed "
+                              "more informative.") + "<b></b>";
 
     titleLabel = new QLabel(tr("Title") + ":");
     titleLabel->setSizePolicy(QSizePolicy::Minimum,
                               QSizePolicy::Maximum);
+    titleLabel->setToolTip(titleTooltip);
 
     titleLineEdit = new QLineEdit();
     titleLineEdit->setSizePolicy(QSizePolicy::MinimumExpanding,
                                  QSizePolicy::Maximum);
     titleLineEdit->setPlaceholderText(tr("Title for the post"));
-    titleLineEdit->setToolTip(tr("Title for the post. Setting a title "
-                                 "helps make the Meanwhile feed "
-                                 "more informative.") + "<b></b>");
+    titleLineEdit->setToolTip(titleTooltip);
 
 
     pictureLabel = new QLabel();
     pictureLabel->setAlignment(Qt::AlignCenter);
-    //pictureLabel->setScaledContents(true);
     pictureLabel->setFrameStyle(QFrame::StyledPanel | QFrame::Raised);
     pictureLabel->hide();
 
 
     mediaInfoLabel = new QLabel();
+    mediaInfoLabel->setWordWrap(true);
     mediaInfoLabel->setAlignment(Qt::AlignTop | Qt::AlignLeft);
 
     selectMediaButton = new QPushButton();
     selectMediaButton->setSizePolicy(QSizePolicy::MinimumExpanding,
-                                       QSizePolicy::Maximum);
+                                     QSizePolicy::Maximum);
+    connect(selectMediaButton, SIGNAL(clicked()),
+            this, SLOT(findMediaFile()));
     selectMediaButton->hide();
 
 
+    uploadProgressBar = new QProgressBar();
 
     // Set default pixmap and "media not set" message
     this->setEmptyMediaData();
@@ -89,9 +94,12 @@ Publisher::Publisher(PumpController *pumpController,
     composerBox = new Composer(true); // forPublisher = true
     composerBox->setSizePolicy(QSizePolicy::Minimum,
                                QSizePolicy::Minimum);
-    connect(composerBox, SIGNAL(focusReceived()),    this, SLOT(setFullMode()));
-    connect(composerBox, SIGNAL(editingFinished()),  this, SLOT(sendPost()));
-    connect(composerBox, SIGNAL(editingCancelled()), this, SLOT(setMinimumMode()));
+    connect(composerBox, SIGNAL(focusReceived()),
+            this, SLOT(setFullMode()));
+    connect(composerBox, SIGNAL(editingFinished()),
+            this, SLOT(sendPost()));
+    connect(composerBox, SIGNAL(editingCancelled()),
+            this, SLOT(setMinimumMode()));
 
     // Add formatting button exported from Composer
     this->toolsButton = composerBox->getToolsButton();
@@ -191,23 +199,28 @@ Publisher::Publisher(PumpController *pumpController,
             this, SLOT(setPictureMode()));
 
     addMediaAudioAction = new QAction(QIcon::fromTheme("audio-input-microphone"),
-                                      tr("Audio") + " (experimental)", this);
-    addMediaAudioAction->setDisabled(true); // Disabled for v1.2.0
+                                      tr("Audio"), this);
     connect(addMediaAudioAction, SIGNAL(triggered()),
             this, SLOT(setAudioMode()));
 
     addMediaVideoAction = new QAction(QIcon::fromTheme("camera-web"),
-                                      tr("Video") + " (experimental)", this);
-    addMediaVideoAction->setDisabled(true); // Disabled for v1.2.0
+                                      tr("Video"), this);
     connect(addMediaVideoAction, SIGNAL(triggered()),
             this, SLOT(setVideoMode()));
 
+    addMediaFileAction = new QAction(QIcon::fromTheme("application-octet-stream"),
+                                     tr("Other", "as in other kinds of files"),
+                                     this);
+    connect(addMediaFileAction, SIGNAL(triggered()),
+            this, SLOT(setFileMode()));
+
 
     // The menu itself
     addMediaMenu = new QMenu();
     addMediaMenu->addAction(addMediaImageAction);
     addMediaMenu->addAction(addMediaAudioAction);
     addMediaMenu->addAction(addMediaVideoAction);
+    addMediaMenu->addAction(addMediaFileAction);
 
     // The "add..." button holding the menu
     addMediaButton = new QPushButton(QIcon::fromTheme("mail-attachment"),
@@ -218,7 +231,7 @@ Publisher::Publisher(PumpController *pumpController,
 
     // Info label for "sending" and other status information
     statusInfoLabel = new QLabel();
-    statusInfoLabel->setSizePolicy(QSizePolicy::MinimumExpanding,
+    statusInfoLabel->setSizePolicy(QSizePolicy::Maximum,
                                    QSizePolicy::Maximum);
     statusInfoLabel->setAlignment(Qt::AlignCenter);
     statusInfoLabel->setWordWrap(true);
@@ -257,7 +270,8 @@ Publisher::Publisher(PumpController *pumpController,
     mainLayout->addLayout(titleLayout,         0, 0, 1, 8, Qt::AlignTop | Qt::AlignLeft);
     mainLayout->addWidget(pictureLabel,        1, 0, 2, 4);
     mainLayout->addWidget(mediaInfoLabel,      1, 4, 1, 4, Qt::AlignTop);
-    mainLayout->addWidget(selectMediaButton,   2, 4, 1, 4, Qt::AlignBottom | Qt::AlignLeft);
+    mainLayout->addWidget(selectMediaButton,   2, 4, 1, 2, Qt::AlignBottom | Qt::AlignLeft);
+    mainLayout->addWidget(uploadProgressBar,   2, 6, 1, 2, Qt::AlignBottom | Qt::AlignRight);
 
     mainLayout->addWidget(composerBox,         3, 0, 3, 8); // 3 rows, 8 columns
 
@@ -441,6 +455,31 @@ void Publisher::setMediaModeWidgets()
     this->selectMediaButton->show();
 }
 
+/*
+ * Disable some widgets while sending a post,
+ * including during media upload
+ *
+ */
+void Publisher::toggleWidgetsWhileSending(bool widgetsEnabled)
+{
+    this->titleLineEdit->setEnabled(widgetsEnabled);
+    this->composerBox->setEnabled(widgetsEnabled);
+
+    this->toolsButton->setEnabled(widgetsEnabled);
+
+    this->toSelectorButton->setEnabled(widgetsEnabled);
+    this->ccSelectorButton->setEnabled(widgetsEnabled);
+
+    // Only re-enable "add" button if this was still a simple note
+    if (postType == "note")
+    {
+        this->addMediaButton->setEnabled(widgetsEnabled);
+    }
+    this->selectMediaButton->setEnabled(widgetsEnabled);
+
+    this->postButton->setEnabled(widgetsEnabled);
+}
+
 
 
 /********************************************************************/
@@ -469,7 +508,9 @@ void Publisher::setMinimumMode()
     this->setMaximumHeight(composerHeight + 2);
 
 
+    this->audienceSelectorTo->deletePrevious();
     this->audienceSelectorTo->resetLists();
+    this->audienceSelectorCC->deletePrevious();
     this->audienceSelectorCC->resetLists();
 
     this->toAudienceLabel->setText("...");
@@ -540,7 +581,7 @@ void Publisher::setMinimumMode()
     this->pictureLabel->hide();
     this->mediaInfoLabel->hide();
     this->selectMediaButton->hide();
-    this->selectMediaButton->disconnect(); // Disconnect from findPicture, findAudio...
+    this->uploadProgressBar->hide();
 
     this->setEmptyMediaData();
 
@@ -606,6 +647,8 @@ void Publisher::setFullMode()
 
 void Publisher::setPictureMode()
 {
+    postType = "image";
+
     setMediaModeWidgets();
 
     pictureLabel->setPixmap(QIcon::fromTheme("image-x-generic",
@@ -618,19 +661,16 @@ void Publisher::setPictureMode()
     selectMediaButton->setIcon(QIcon::fromTheme("folder-image"));
     selectMediaButton->setText(tr("Select Picture..."));
     selectMediaButton->setToolTip(tr("Find the picture in your folders"));
-    connect(selectMediaButton, SIGNAL(clicked()),
-            this, SLOT(findPictureFile()));
 
     pictureLabel->setToolTip(tr("Picture not set"));
 
-    postType = "image";
-
-    this->findPictureFile(); // Show the open file dialog directly
+    this->findMediaFile(); // Show the open file dialog directly
 }
 
 
 void Publisher::setAudioMode()
 {
+    postType = "audio";
     setMediaModeWidgets();
 
     pictureLabel->setPixmap(QIcon::fromTheme("audio-x-generic",
@@ -643,19 +683,17 @@ void Publisher::setAudioMode()
     selectMediaButton->setIcon(QIcon::fromTheme("folder-sound"));
     selectMediaButton->setText(tr("Select Audio File..."));
     selectMediaButton->setToolTip(tr("Find the audio file in your folders"));
-    connect(selectMediaButton, SIGNAL(clicked()),
-            this, SLOT(findAudioFile()));
 
     pictureLabel->setToolTip(tr("Audio file not set"));
 
-    postType = "audio";
-
-    this->findAudioFile(); // Show the open file dialog directly
+    this->findMediaFile(); // Show the open file dialog directly
 }
 
 
 void Publisher::setVideoMode()
 {
+    postType = "video";
+
     setMediaModeWidgets();
 
     pictureLabel->setPixmap(QIcon::fromTheme("video-x-generic",
@@ -668,14 +706,32 @@ void Publisher::setVideoMode()
     selectMediaButton->setIcon(QIcon::fromTheme("folder-video"));
     selectMediaButton->setText(tr("Select Video..."));
     selectMediaButton->setToolTip(tr("Find the video in your folders"));
-    connect(selectMediaButton, SIGNAL(clicked()),
-            this, SLOT(findVideoFile()));
 
     pictureLabel->setToolTip(tr("Video not set"));
 
-    postType = "video";
+    this->findMediaFile(); // Show the open file dialog directly
+}
 
-    this->findVideoFile(); // Show the open file dialog directly
+void Publisher::setFileMode()
+{
+    postType = "file";
+
+    setMediaModeWidgets();
+
+    pictureLabel->setPixmap(QIcon::fromTheme("application-octet-stream",
+                                             QIcon(":/images/attached-file.png"))
+                            .pixmap(200, 150)
+                            .scaled(200, 150,
+                                    Qt::IgnoreAspectRatio,
+                                    Qt::SmoothTransformation));
+
+    selectMediaButton->setIcon(QIcon::fromTheme("folder"));
+    selectMediaButton->setText(tr("Select File..."));
+    selectMediaButton->setToolTip(tr("Find the file in your folders"));
+
+    pictureLabel->setToolTip(tr("File not set"));
+
+    this->findMediaFile(); // Show the open file dialog directly
 }
 
 
@@ -724,7 +780,7 @@ void Publisher::setEditingMode(QString postID, QString postText,
 void Publisher::onPublishingOk()
 {
     this->statusInfoLabel->clear();
-    this->setEnabled(true);
+    this->toggleWidgetsWhileSending(true);
 
     this->composerBox->erase();
 
@@ -740,7 +796,7 @@ void Publisher::onPublishingFailed()
 {
     qDebug() << "Posting failed, re-enabling Publisher";
     this->statusInfoLabel->setText(tr("Posting failed.\n\nTry again."));
-    this->setEnabled(true);
+    this->toggleWidgetsWhileSending(true);
     this->composerBox->setFocus();
 }
 
@@ -935,10 +991,20 @@ void Publisher::sendPost()
 {
     qDebug() << "Publisher character count:" << composerBox->textCursor().document()->characterCount();
 
+    bool textIsEmpty;
+    if (composerBox->textCursor().document()->characterCount() > 1) // kinda tmp
+    {
+        textIsEmpty = false;
+    }
+    else
+    {
+        textIsEmpty = true;
+    }
+
 
     // If there's some text in the post, or attached media, send it
-    if (composerBox->textCursor().document()->characterCount() > 1     // kinda tmp
-      || !mediaFilename.isEmpty())
+    if ( (postType == "note" && !textIsEmpty)
+      || (postType != "note" && !mediaFilename.isEmpty()))
     {
         QString postTitle = this->titleLineEdit->text().trimmed();
         postTitle.replace("\n", " "); // Post title could have newlines if copy-pasted
@@ -950,7 +1016,7 @@ void Publisher::sendPost()
 
 
         // Don't erase just yet!! Just disable until we get "200 OK" from the server.
-        this->setDisabled(true);
+        this->toggleWidgetsWhileSending(false);
 
         if (!editingMode)
         {
@@ -964,12 +1030,19 @@ void Publisher::sendPost()
             }
             else
             {
-                this->pController->postMedia(audienceMap,
-                                             cleanHtmlString,
-                                             postTitle,
-                                             mediaFilename,
-                                             postType,
-                                             mediaContentType);
+                uploadNetworkReply =  this->pController->postMedia(audienceMap,
+                                                                   cleanHtmlString,
+                                                                   postTitle,
+                                                                   mediaFilename,
+                                                                   postType,
+                                                                   mediaContentType);
+                connect(uploadNetworkReply, SIGNAL(uploadProgress(qint64,qint64)),
+                        this, SLOT(updateProgressBar(qint64,qint64)));
+                // This will be automatically disconnected when
+                // the networkReply is automatically deleted!
+
+                uploadProgressBar->setValue(0);
+                uploadProgressBar->show();
             }
         }
         else
@@ -982,25 +1055,73 @@ void Publisher::sendPost()
     }
     else
     {
-        this->statusInfoLabel->setText(tr("Post is empty."));
-        qDebug() << "Can't send post: text is empty, and no media";
+        if (this->postType == "note")
+        {
+            this->statusInfoLabel->setText(tr("Post is empty."));
+        }
+        else // image, audio...
+        {
+            this->statusInfoLabel->setText(tr("File not selected."));
+        }
+        qDebug() << "Can't send post: text is empty, or no media";
     }
 }
 
 
 
 /*
- * Let the user find an image file in his/her folders, for picture upload
+ * Let the user find a file in his/her folders, for media upload
  *
  */
-void Publisher::findPictureFile()
+void Publisher::findMediaFile()
 {
+    QString findDialogTitle;
+    QString mediaTypes;
+    QString errorTitle;
+    QString errorMessage;
+
+    if (postType == "image")
+    {
+        findDialogTitle = tr("Select one image");
+        mediaTypes = tr("Image files") + "(*.png *.jpg *.jpeg *.gif);;";
+
+        errorTitle =  tr("Invalid image");
+        errorMessage = tr("The image format cannot be detected.\n"
+                          "The extension might be wrong, like a GIF "
+                          "image renamed to image.jpg or similar.");
+    }
+    else if (postType == "audio")
+    {
+        findDialogTitle = tr("Select one audio file");
+        mediaTypes = tr("Audio files") + "(*.ogg *.flac *.mp3 *.wav);;";
+
+        errorTitle = tr("Invalid audio file");
+        errorMessage = tr("The audio format cannot be detected.");
+    }
+    else if (postType == "video")
+    {
+        findDialogTitle = tr("Select one video file");
+        mediaTypes = tr("Video files") + "(*.avi *.mpg *.mpeg *.ogm *.mkv *.mp4);;";
+
+        errorTitle = tr("Invalid video file");
+        errorMessage = tr("The video format cannot be detected.");
+    }
+    else // File
+    {
+        findDialogTitle = tr("Select one file");
+        mediaTypes = "";
+
+        errorTitle = tr("Invalid file");
+        errorMessage = tr("The file type cannot be detected.");
+    }
+
     QString filename;
-    filename = QFileDialog::getOpenFileName(this, tr("Select one image"),
+    filename = QFileDialog::getOpenFileName(this, findDialogTitle,
                                             this->lastUsedDirectory,
-                                            tr("Image files") + "(*.png *.jpg *.jpeg *.gif);;"
+                                            mediaTypes
                                             + tr("All files") + " (*)");
 
+
     if (!filename.isEmpty())
     {
         qDebug() << "Selected" << filename << "for upload";
@@ -1011,119 +1132,56 @@ void Publisher::findPictureFile()
             this->pictureLabel->setToolTip(filename);
             this->mediaFilename = filename;
 
-            QPixmap imagePixmap = QPixmap(filename);
-            // FIXME: check imagePixmap.isNull() ?
-            this->pictureLabel->setPixmap(imagePixmap.scaled(320, 180,  // 16:9
-                                                             Qt::KeepAspectRatio,
-                                                             Qt::SmoothTransformation));
+            QString metaDataString;
+            if (postType == "image")
+            {
+                QPixmap imagePixmap = QPixmap(filename);
+                // FIXME: check imagePixmap.isNull() ?
+                this->pictureLabel->setPixmap(imagePixmap.scaled(320, 180,  // 16:9
+                                                                 Qt::KeepAspectRatio,
+                                                                 Qt::SmoothTransformation));
+                metaDataString = QString("<br><br>"
+                                         "<b>" +tr("Resolution") + ":</b> %1x%2")
+                                         .arg(imagePixmap.width())
+                                         .arg(imagePixmap.height());
+            }
             QFileInfo fileInfo(filename);
             this->lastUsedDirectory = fileInfo.path();
             qDebug() << "last used directory:" << lastUsedDirectory;
 
 
-            mediaInfoLabel->setText(QString("<b>" +tr("Resolution") + ":</b> %1x%2"
-                                      "<br>"
-                                      "<b>" + tr("Size") + ":</b> %3")
-                                      .arg(imagePixmap.width())
-                                      .arg(imagePixmap.height())
-                                      .arg(MiscHelpers::fileSizeString(filename)));
+            mediaInfoLabel->setText(QString("<b>%1</b>"
+                                            "<br>"
+                                            "<b>" + tr("Type") + ":</b> %2"
+                                            "<br>"
+                                            "<b>" + tr("Size") + ":</b> %3")
+                                    .arg(fileInfo.fileName(),
+                                         mediaContentType,
+                                         MiscHelpers::fileSizeString(filename))
+                                    + metaDataString);
         }
         else
         {
-            qDebug() << "Unknown image format; Extension is probably wrong";
-            QMessageBox::warning(this, tr("Invalid image"),
-                                 tr("The image format cannot be detected.\n"
-                                    "The extension might be wrong, like a GIF "
-                                    "image renamed to image.jpg or similar."));
+            qDebug() << "Unknown " << postType << " format; Extension is probably wrong";
+            QMessageBox::warning(this, errorTitle, errorMessage);
 
             this->mediaContentType.clear();
             this->mediaFilename.clear();
             this->pictureLabel->setToolTip("");
         }
+
+
+        this->uploadProgressBar->hide();
     }
 }
 
 
-
-
-void Publisher::findAudioFile()
+void Publisher::updateProgressBar(qint64 sent, qint64 total)
 {
-    QString filename;
-    filename = QFileDialog::getOpenFileName(this, tr("Select one audio file"),
-                                            this->lastUsedDirectory,
-                                            tr("Audio files") + "(*.ogg *.flac *.mp3 *.wav);;"
-                                            + tr("All files") + " (*)");
+    this->uploadProgressBar->setRange(0, total);
+    this->uploadProgressBar->setValue(sent);
 
-    if (!filename.isEmpty())
-    {
-        qDebug() << "Selected" << filename << "for upload";
-
-        this->mediaContentType = MiscHelpers::getFileMimeType(filename);
-        if (!mediaContentType.isEmpty())
-        {
-            this->pictureLabel->setToolTip(filename);
-            this->mediaFilename = filename;
-
-            QFileInfo fileInfo(filename);
-            this->lastUsedDirectory = fileInfo.path();
-            qDebug() << "last used directory:" << lastUsedDirectory;
-
-
-            mediaInfoLabel->setText(QString("<b>" + tr("Size") + ":</b> %1")
-                                    .arg(MiscHelpers::fileSizeString(filename)));
-        }
-        else
-        {
-            qDebug() << "Unknown audio format; Extension is probably wrong";
-            QMessageBox::warning(this, tr("Invalid audio file"),
-                                 tr("The audio format cannot be detected."));
-
-            this->mediaContentType.clear();
-            this->mediaFilename.clear();
-            this->pictureLabel->setToolTip("");
-        }
-    }
-}
-
-
-
-
-void Publisher::findVideoFile()
-{
-    QString filename;
-    filename = QFileDialog::getOpenFileName(this, tr("Select one video file"),
-                                            this->lastUsedDirectory,
-                                            tr("Video files")
-                                            + "(*.avi *.mpg *.mpeg *.ogm *.mkv *.mp4);;"
-                                            + tr("All files") + " (*)");
-
-    if (!filename.isEmpty())
-    {
-        qDebug() << "Selected" << filename << "for upload";
-
-        this->mediaContentType = MiscHelpers::getFileMimeType(filename);
-        if (!mediaContentType.isEmpty())
-        {
-            this->pictureLabel->setToolTip(filename);
-            this->mediaFilename = filename;
-
-            QFileInfo fileInfo(filename);
-            this->lastUsedDirectory = fileInfo.path();
-            qDebug() << "last used directory:" << lastUsedDirectory;
-
-
-            mediaInfoLabel->setText(QString("<b>" + tr("Size") + ":</b> %1")
-                                    .arg(MiscHelpers::fileSizeString(filename)));
-        }
-        else
-        {
-            qDebug() << "Unknown video format; Extension is probably wrong";
-            QMessageBox::warning(this, tr("Invalid video file"),
-                                 tr("The video format cannot be detected."));
-
-            this->mediaContentType.clear();
-            this->mediaFilename.clear();
-            this->pictureLabel->setToolTip("");
-        }
-    }
+    this->uploadProgressBar->setToolTip(tr("%1 KiB of %2 KiB uploaded")
+                                         .arg(sent / 1024)
+                                        .arg(total / 1024));
 }
