@@ -30,6 +30,7 @@
 #include <QMenu>
 #include <QFileDialog>
 #include <QMessageBox>
+#include <QProgressBar>
 
 #include <QDebug>
 
@@ -55,14 +56,8 @@ public:
     QMap<QString,QString> getAudienceMap();
 
     void setMediaModeWidgets();
+    void toggleWidgetsWhileSending(bool widgetsEnabled);
 
-    enum PostTypes
-    {
-        NoteType,
-        ImageType,
-        AudioType,
-        VideoType
-    };
 
 signals:
 
@@ -70,9 +65,11 @@ signals:
 public slots:
     void setMinimumMode();
     void setFullMode();
+
     void setPictureMode();
     void setAudioMode();
     void setVideoMode();
+    void setFileMode();
 
     void setEditingMode(QString postID,
                         QString postText,
@@ -96,9 +93,9 @@ public slots:
 
     void sendPost();
 
-    void findPictureFile();
-    void findAudioFile();
-    void findVideoFile();
+    void findMediaFile();
+
+    void updateProgressBar(qint64 sent, qint64 total);
 
 
 private:
@@ -110,6 +107,7 @@ private:
     QLabel *mediaInfoLabel;
     QPushButton *selectMediaButton;
     QLabel *pictureLabel;
+    QProgressBar *uploadProgressBar;
 
     QPushButton *toolsButton;
 
@@ -151,6 +149,7 @@ private:
     QAction *addMediaImageAction;
     QAction *addMediaAudioAction;
     QAction *addMediaVideoAction;
+    QAction *addMediaFileAction;
 
     QLabel *statusInfoLabel;
 
@@ -166,6 +165,8 @@ private:
     bool editingMode;
     QString editingPostId;
 
+    QNetworkReply *uploadNetworkReply;
+
     PumpController *pController;
 };
 
