@@ -22,10 +22,7 @@
 
 PumpController::PumpController(QObject *parent) :  QObject(parent)
 {
-    connect(&nam, SIGNAL(destroyed()),
-            this, SLOT(tmpfixmeNotifyQNAMdestroyed()));
-
-    this->userAgentString = "Dianara/1.2.0";
+    this->userAgentString = "Dianara/1.2.1";
 
     this->postsPerPageMain = 20;
     this->postsPerPageOther = 10;
@@ -344,6 +341,17 @@ void PumpController::getImage(QString imageUrl)
 }
 
 
+QNetworkReply *PumpController::getMedia(QString mediaUrl)
+{
+    QNetworkRequest mediaRequest = this->prepareRequest(mediaUrl,
+                                                        QOAuth::GET,
+                                                        MediaRequest);
+
+    qDebug() << "getMedia() sending mediaRequest for:" << mediaUrl;
+    return nam.get(mediaRequest);
+}
+
+
 
 void PumpController::notifyAvatarStored(QString avatarUrl, QString avatarFilename)
 {
@@ -921,9 +929,9 @@ QNetworkRequest PumpController::prepareRequest(QString url, QOAuth::HttpMethod m
  * Used to upload pictures, but might have more uses in the future
  *
  */
-void PumpController::uploadFile(QString filename,
-                                QString contentType,
-                                int uploadType)
+QNetworkReply *PumpController::uploadFile(QString filename,
+                                          QString contentType,
+                                          int uploadType)
 {
     qDebug() << "PumpController::uploadFile()";
 
@@ -936,11 +944,18 @@ void PumpController::uploadFile(QString filename,
 
     QFile file(filename);
     file.open(QIODevice::ReadOnly);
-
-    QByteArray data = file.readAll();
+    QByteArray data = file.readAll();  // FIXME: This can use a lot of RAM with big files
     file.close();
 
-    nam.post(postRequest, data);
+    QString prettyLogMessage = tr("Uploading %1",
+                                  "1=filename").arg(filename)
+                               + QString(" (%1, %2)")
+                                 .arg(contentType)
+                                 .arg(MiscHelpers::fileSizeString(filename));
+    emit currentJobChanged(prettyLogMessage);
+    emit logMessage(prettyLogMessage);
+
+    return nam.post(postRequest, data);
 }
 
 
@@ -1058,6 +1073,14 @@ void PumpController::requestFinished(QNetworkReply *reply)
         {
             emit commentPostingFailed(); // also, kinda TMP
         }
+        else if (requestType == UploadMediaForPostRequest)
+        {
+            emit postPublishingFailed(); // FIXME?
+        }
+        else if (requestType == MediaRequest)
+        {
+            emit downloadFailed(replyUrl);
+        }
     }
 
 
@@ -1212,8 +1235,11 @@ void PumpController::requestFinished(QNetworkReply *reply)
     QVariantList jsonDataList;
 
 
-    // Unless it was an AvatarRequest or an ImageRequest, it should be JSON, so parse it
-    if (requestType != AvatarRequest && requestType != ImageRequest)
+    // Unless it was an AvatarRequest, ImageRequest or MediaRequest,
+    // it should be JSON, so parse it
+    if (requestType != AvatarRequest
+     && requestType != ImageRequest
+     && requestType != MediaRequest)
     {
         jsonData = jsonParser.parse(replyData, &jsonParsedOK).toMap();
         qDebug() << "JSON data size (items):" << jsonData.size();
@@ -1372,7 +1398,8 @@ void PumpController::requestFinished(QNetworkReply *reply)
             QString objectType = jsonData["object"].toMap().value("objectType").toString();
             if (objectType == "image"
              || objectType == "audio"
-             || objectType == "video")
+             || objectType == "video"
+             || objectType == "file")
             {
                 // Update the object with title and description
                 this->postMediaStepThree(objectID);
@@ -1789,6 +1816,25 @@ void PumpController::requestFinished(QNetworkReply *reply)
         break;
 
 
+    case MediaRequest:
+        qDebug() << "Received MEDIA data, from " << replyUrl;
+        if (finished)
+        {
+            prettyLogMessage = tr("File downloaded successfully.");
+            emit currentJobChanged(prettyLogMessage);
+            emit logMessage(prettyLogMessage, replyUrl);
+
+            emit downloadCompleted(replyUrl);
+
+            qDebug() << "Media received 100%" << replyData.length() << "bytes";
+        }
+        else
+        {
+            qDebug() << "Media not complete yet";
+        }
+        break;
+
+
     //////////////////////////////////////// If uploading a file was requested
     case UploadAvatarRequest:
         // just jump to next
@@ -1801,24 +1847,28 @@ void PumpController::requestFinished(QNetworkReply *reply)
         {
             qDebug() << "JSON parsed OK";
 
-            QString uploadedFileID = jsonData["id"].toString();
-            qDebug() << "Uploaded file ID:" << uploadedFileID;
+            QString uploadedFileId = jsonData["id"].toString();
+            qDebug() << "Uploaded file ID:" << uploadedFileId;
 
 
             QString objectType = jsonData["objectType"].toString();
             if (objectType == "image"
              || objectType == "audio"
-             || objectType == "video")
+             || objectType == "video"
+             || objectType == "file")
             {
                 if (requestType == UploadMediaForPostRequest)
                 {
-                    emit currentJobChanged(tr("File uploaded successfully. Posting message..."));
-                    this->postMediaStepTwo(uploadedFileID);
+                    prettyLogMessage = tr("File uploaded successfully. Posting message...");
+                    emit currentJobChanged(prettyLogMessage);
+                    emit logMessage(prettyLogMessage, replyUrl);
+
+                    this->postMediaStepTwo(uploadedFileId);
                 }
                 else if (requestType == UploadAvatarRequest)
                 {
                     emit currentJobChanged(tr("Avatar uploaded."));
-                    this->postAvatarStepTwo(uploadedFileID);
+                    this->postAvatarStepTwo(uploadedFileId);
                 }
             }
         }
@@ -1866,6 +1916,8 @@ void PumpController::sslErrorsHandler(QNetworkReply *reply, QList<QSslError> err
         }
     }
     */
+
+    reply->deleteLater();
 }
 
 
@@ -2031,8 +2083,8 @@ void PumpController::authorizeApplication(QString verifierCode)
 
 
 /*
- * Called by a QTimer, get initial data (profile, contacts, timelines),
- * one step at a time
+ * Called by a QTimer;
+ * Get initial data (profile, contacts, timelines), one step at a time
  *
  */
 void PumpController::getInitialData()
@@ -2067,11 +2119,11 @@ void PumpController::getInitialData()
         break;
 
     case 3:
-        this->getMainTimeline(0);
+        // Nothing, just extra time to get 'following' before timelines
         break;
 
     case 4:
-        this->getListsList();
+        this->getMainTimeline(0);
         break;
 
     case 5:
@@ -2079,18 +2131,22 @@ void PumpController::getInitialData()
         break;
 
     case 6:
-        this->getDirectTimeline(0);
+        this->getListsList();
         break;
 
     case 7:
-        this->getActivityTimeline(0);
+        this->getDirectTimeline(0);
         break;
 
     case 8:
-        this->getFavoritesTimeline(0);
+        this->getActivityTimeline(0);
         break;
 
     case 9:
+        this->getFavoritesTimeline(0);
+        break;
+
+    case 10:
         // Do nothing, so it takes longer for the final (default) step to arrive
         break;
 
@@ -2122,8 +2178,9 @@ void PumpController::postNote(QMap<QString, QString> audienceMap,
 
     QString url = "https://" + this->serverURL + "/api/user/" + this->userName + "/feed";
 
-    QNetworkRequest postRequest = this->prepareRequest(url, QOAuth::POST, PublishPostRequest);
-
+    QNetworkRequest postRequest = this->prepareRequest(url,
+                                                       QOAuth::POST,
+                                                       PublishPostRequest);
 
     qDebug() << "Should be posting to:" << audienceMap;
 
@@ -2167,11 +2224,10 @@ void PumpController::postNote(QMap<QString, QString> audienceMap,
  * Then we get it's ID in a signal, and create the post itself
  *
  */
-void PumpController::postMedia(QMap<QString, QString> audienceMap,
-                               QString postText, QString postTitle,
-                               QString mediaFilename,
-                               QString mediaType,
-                               QString mimeContentType)
+QNetworkReply *PumpController::postMedia(QMap<QString, QString> audienceMap,
+                                         QString postText, QString postTitle,
+                                         QString mediaFilename, QString mediaType,
+                                         QString mimeContentType)
 {
     qDebug() << "PumpController::postMedia()";
     qDebug() << "Uploading" << mediaFilename << "with title:" << postTitle;
@@ -2182,8 +2238,11 @@ void PumpController::postMedia(QMap<QString, QString> audienceMap,
     this->currentAudienceMap = audienceMap;
     this->currentPostType = mediaType;
 
-
-    this->uploadFile(mediaFilename, mimeContentType, UploadMediaForPostRequest);
+    QNetworkReply *networkReply;
+    networkReply =  this->uploadFile(mediaFilename,
+                                     mimeContentType,
+                                     UploadMediaForPostRequest);
+    return networkReply;
 }
 
 
@@ -2199,7 +2258,9 @@ void PumpController::postMediaStepTwo(QString id)
 
     QString url = "https://" + this->serverURL + "/api/user/" + this->userName + "/feed";
 
-    QNetworkRequest postRequest = this->prepareRequest(url, QOAuth::POST, PublishPostRequest);
+    QNetworkRequest postRequest = this->prepareRequest(url,
+                                                       QOAuth::POST,
+                                                       PublishPostRequest);
 
 
     QVariantMap jsonVariantObject;
@@ -2234,7 +2295,9 @@ void PumpController::postMediaStepThree(QString id)
     qDebug() << "PumpController::postMediaStepThree() post ID:" << id;
 
     // Using the ID directly as URL
-    QNetworkRequest postRequest = this->prepareRequest(id, QOAuth::PUT, UpdatePostRequest);
+    QNetworkRequest postRequest = this->prepareRequest(id,
+                                                       QOAuth::PUT,
+                                                       UpdatePostRequest);
 
     QVariantMap jsonVariant;
     jsonVariant.insert("verb", "update"); // "post" would suffice
@@ -2306,7 +2369,9 @@ void PumpController::updatePost(QString id,
     qDebug() << "PumpController::updatePost(), post ID:" << id;
 
     // Using the ID directly as URL
-    QNetworkRequest postRequest = this->prepareRequest(id, QOAuth::PUT, UpdatePostRequest);
+    QNetworkRequest postRequest = this->prepareRequest(id,
+                                                       QOAuth::PUT,
+                                                       UpdatePostRequest);
 
     QVariantMap jsonVariant;
     jsonVariant.insert("verb", "update");     // "post" would suffice
@@ -2332,7 +2397,9 @@ void PumpController::likePost(QString postID, QString postType, bool like)
 
     QString url =  "https://" + this->serverURL + "/api/user/" + this->userName + "/feed";
 
-    QNetworkRequest likeRequest = this->prepareRequest(url, QOAuth::POST, LikePostRequest);
+    QNetworkRequest likeRequest = this->prepareRequest(url,
+                                                       QOAuth::POST,
+                                                       LikePostRequest);
 
 
     QVariantMap jsonVariantObject;
@@ -2359,9 +2426,9 @@ void PumpController::addComment(QString comment, QString postID, QString postTyp
 
     QString url =  "https://" + this->serverURL + "/api/user/" + this->userName + "/feed";
 
-    QNetworkRequest commentRequest = this->prepareRequest(url, QOAuth::POST, CommentPostRequest);
-
-
+    QNetworkRequest commentRequest = this->prepareRequest(url,
+                                                          QOAuth::POST,
+                                                          CommentPostRequest);
 
     QVariantMap jsonVariantInReplyTo;
     jsonVariantInReplyTo.insert("id", postID);
@@ -2394,7 +2461,8 @@ void PumpController::updateComment(QString id, QString content)
     qDebug() << "PumpController::updateComment(), comment ID:" << id;
 
     // Using the ID directly as URL
-    QNetworkRequest postRequest = this->prepareRequest(id, QOAuth::PUT,
+    QNetworkRequest postRequest = this->prepareRequest(id,
+                                                       QOAuth::PUT,
                                                        UpdateCommentRequest);
 
     QVariantMap jsonVariant;
@@ -2417,8 +2485,9 @@ void PumpController::sharePost(QString postID, QString postType)
 
     QString url =  "https://" + this->serverURL + "/api/user/" + this->userName + "/feed";
 
-    QNetworkRequest shareRequest = this->prepareRequest(url, QOAuth::POST, SharePostRequest);
-
+    QNetworkRequest shareRequest = this->prepareRequest(url,
+                                                        QOAuth::POST,
+                                                        SharePostRequest);
 
     QVariantMap jsonVariantObject;
     jsonVariantObject.insert("objectType", postType);
@@ -2443,8 +2512,9 @@ void PumpController::unsharePost(QString postId, QString postType)
 
     QString url =  "https://" + this->serverURL + "/api/user/" + this->userName + "/feed";
 
-    QNetworkRequest unshareRequest = this->prepareRequest(url, QOAuth::POST, UnsharePostRequest);
-
+    QNetworkRequest unshareRequest = this->prepareRequest(url,
+                                                          QOAuth::POST,
+                                                          UnsharePostRequest);
 
     QVariantMap jsonVariantObject;
     jsonVariantObject.insert("objectType", postType);
@@ -2470,9 +2540,9 @@ void PumpController::deletePost(QString postID, QString postType)
 
     QString url =  "https://" + this->serverURL + "/api/user/" + this->userName + "/feed";
 
-    QNetworkRequest deleteRequest = this->prepareRequest(url, QOAuth::POST, DeletePostRequest);
-
-
+    QNetworkRequest deleteRequest = this->prepareRequest(url,
+                                                         QOAuth::POST,
+                                                         DeletePostRequest);
     QVariantMap jsonVariantObject;
     jsonVariantObject.insert("objectType", postType);
     jsonVariantObject.insert("id", postID);
@@ -2505,9 +2575,9 @@ void PumpController::followContact(QString address)
 
     QString url =  "https://" + this->serverURL + "/api/user/" + this->userName + "/feed";
 
-    QNetworkRequest followRequest = this->prepareRequest(url, QOAuth::POST, FollowContactRequest);
-
-
+    QNetworkRequest followRequest = this->prepareRequest(url,
+                                                         QOAuth::POST,
+                                                         FollowContactRequest);
     QVariantMap jsonVariantObject;
     jsonVariantObject.insert("objectType", "person");
     jsonVariantObject.insert("id", "acct:" + address);
@@ -2539,9 +2609,9 @@ void PumpController::unfollowContact(QString address)
 
     QString url =  "https://" + this->serverURL + "/api/user/" + this->userName + "/feed";
 
-    QNetworkRequest unfollowRequest = this->prepareRequest(url, QOAuth::POST, UnfollowContactRequest);
-
-
+    QNetworkRequest unfollowRequest = this->prepareRequest(url,
+                                                           QOAuth::POST,
+                                                           UnfollowContactRequest);
     QVariantMap jsonVariantObject;
     jsonVariantObject.insert("objectType", "person");
     jsonVariantObject.insert("id", "acct:" + address);
@@ -2557,8 +2627,3 @@ void PumpController::unfollowContact(QString address)
 
     nam.post(unfollowRequest, data);
 }
-
-void PumpController::tmpfixmeNotifyQNAMdestroyed()
-{
-    qDebug() << "QNAM DESTROYED!!! ########################################\n\n\n\nBOOM!!\n\n";
-}
