@@ -84,6 +84,7 @@ public:
     void enqueueImageForDownload(QString url);
     void getAvatar(QString avatarUrl);
     void getImage(QString imageUrl);
+    QNetworkReply *getMedia(QString mediaUrl);
     void notifyAvatarStored(QString avatarUrl, QString avatarFilename);
     void notifyImageStored(QString imageUrl);
 
@@ -118,8 +119,8 @@ public:
                                    QOAuth::ParamMap paramMap = QOAuth::ParamMap(),
                                    QString contentTypeString="application/json");
 
-    void uploadFile(QString filename, QString contentType,
-                    int uploadType = UploadFileRequest);
+    QNetworkReply *uploadFile(QString filename, QString contentType,
+                              int uploadType = UploadFileRequest);
 
     QList<QVariantList> processAudience(QMap<QString,QString> audienceMap);
 
@@ -171,6 +172,7 @@ public:
 
         AvatarRequest,
         ImageRequest,
+        MediaRequest,
 
         UploadFileRequest,
         UploadMediaForPostRequest,
@@ -215,6 +217,8 @@ signals:
 
     void avatarPictureReceived(QByteArray pictureData, QUrl pictureURL);
     void imageReceived(QByteArray pictureData, QUrl pictureURL);
+    void downloadCompleted(QString fileUrl);
+    void downloadFailed(QString fileUrl);
     void avatarStored(QString avatarUrl, QString avatarFilename);
     void imageStored(QString imageUrl);
 
@@ -245,10 +249,10 @@ public slots:
     void postNote(QMap<QString,QString> audienceMap,
                   QString postText,
                   QString postTitle);
-    void postMedia(QMap<QString,QString> audienceMap,
-                   QString postText, QString postTitle,
-                   QString mediaFilename, QString mediaType,
-                   QString mimeContentType);
+    QNetworkReply *postMedia(QMap<QString,QString> audienceMap,
+                             QString postText, QString postTitle,
+                             QString mediaFilename, QString mediaType,
+                             QString mimeContentType);
     void postMediaStepTwo(QString id);
     void postMediaStepThree(QString id);
 
@@ -269,7 +273,6 @@ public slots:
     void followContact(QString address);
     void unfollowContact(QString address);
 
-    void tmpfixmeNotifyQNAMdestroyed(); // FIXME TMP KILL IT!
 
 private:
     QNetworkAccessManager nam;
