@@ -22,10 +22,18 @@
 
 TimeLine::TimeLine(int timelineType,
                    PumpController *pumpController,
+                   FilterChecker *filterChecker,
                    QWidget *parent) :  QWidget(parent)
 {
     this->timelineType = timelineType;
     this->pController = pumpController;
+    this->fChecker = filterChecker;
+
+    // Set 5 empty colors initially
+    this->highlightColorsList << "" << "" << "" << "" << "";
+    // They will be set later, but the string list needs to have all 5 elements
+    // before the demo posts are created
+
 
     // Simulated data for demo posts
     QVariantMap demoLocationData;
@@ -136,47 +144,97 @@ TimeLine::TimeLine(int timelineType,
 
     this->minMaxHeightForPosts = 400;
 
-    firstPageButton = new QPushButton(QIcon::fromTheme("go-first"), tr("F&irst Page"));
+    firstPageButton = new QPushButton(QIcon::fromTheme("go-first"), tr("&Newest"));
     connect(firstPageButton, SIGNAL(clicked()),
             this, SLOT(goToFirstPage()));
 
     currentPageLabel = new QLabel();
     currentPageLabel->setFrameStyle(QFrame::StyledPanel | QFrame::Sunken);
-    currentPageLabel->setSizePolicy(QSizePolicy::Maximum,
+    currentPageLabel->setSizePolicy(QSizePolicy::Minimum,
                                     QSizePolicy::Maximum);
 
-    previousPageButton = new QPushButton(QIcon::fromTheme("go-previous"), tr("&Previous Page"));
+    previousPageButton = new QPushButton(QIcon::fromTheme("go-previous"), tr("Newer &Posts"));
     connect(previousPageButton, SIGNAL(clicked()),
             this, SLOT(goToPreviousPage()));
-    nextPageButton = new QPushButton(QIcon::fromTheme("go-next"), tr("&Next Page"));
+    nextPageButton = new QPushButton(QIcon::fromTheme("go-next"), tr("&Older Posts"));
     connect(nextPageButton, SIGNAL(clicked()),
             this, SLOT(goToNextPage()));
 
 
     ///// Layout
-    mainLayout = new QVBoxLayout();
-
     postsLayout = new QVBoxLayout();
+    postsLayout->setContentsMargins(0, 0, 0, 0);
     postsLayout->setAlignment(Qt::AlignTop);
-    mainLayout->addLayout(postsLayout, 1);
-
-    mainLayout->addSpacing(2); // 2 pixel separation
 
     bottomLayout = new QHBoxLayout();
-    bottomLayout->addWidget(firstPageButton);
-    bottomLayout->addSpacing(8);
-    bottomLayout->addWidget(currentPageLabel);
-    bottomLayout->addSpacing(8);
-    bottomLayout->addWidget(previousPageButton);
-    bottomLayout->addWidget(nextPageButton);
+    bottomLayout->addSpacing(2);
+    bottomLayout->addWidget(firstPageButton,    2);
+    bottomLayout->addSpacing(4);
+    bottomLayout->addWidget(currentPageLabel,   0);
+    bottomLayout->addSpacing(4);
+    bottomLayout->addWidget(previousPageButton, 2);
+    bottomLayout->addWidget(nextPageButton,     2);
+    bottomLayout->addSpacing(2);
+
+
+    mainLayout = new QVBoxLayout();
+    mainLayout->setContentsMargins(0, 0, 0, 0);
+    mainLayout->addLayout(postsLayout, 1);
+    mainLayout->addSpacing(2);              // 2 pixel separation
     mainLayout->addLayout(bottomLayout, 0);
 
     this->setLayout(mainLayout);
 
 
+    // QActions for better keyboard control, single step
+    scrollUpAction = new QAction(this);
+    scrollUpAction->setShortcut(QKeySequence("Ctrl+Up"));
+    connect(scrollUpAction, SIGNAL(triggered()),
+            this, SLOT(scrollUp()));
+    this->addAction(scrollUpAction);
+
+    scrollDownAction = new QAction(this);
+    scrollDownAction->setShortcut(QKeySequence("Ctrl+Down"));
+    connect(scrollDownAction, SIGNAL(triggered()),
+            this, SLOT(scrollDown()));
+    this->addAction(scrollDownAction);
+
+    // Pages
+    scrollPageUpAction = new QAction(this);
+    scrollPageUpAction->setShortcut(QKeySequence("Ctrl+PgUp"));
+    connect(scrollPageUpAction, SIGNAL(triggered()),
+            this, SLOT(scrollPageUp()));
+    this->addAction(scrollPageUpAction);
+
+    scrollPageDownAction = new QAction(this);
+    scrollPageDownAction->setShortcut(QKeySequence("Ctrl+PgDown"));
+    connect(scrollPageDownAction, SIGNAL(triggered()),
+            this, SLOT(scrollPageDown()));
+    this->addAction(scrollPageDownAction);
+
+    // Top / Bottom
+    scrollTopAction = new QAction(this);
+    scrollTopAction->setShortcut(QKeySequence("Ctrl+Home"));
+    connect(scrollTopAction, SIGNAL(triggered()),
+            this, SLOT(scrollToTop()));
+    this->addAction(scrollTopAction);
+
+    scrollBottomAction = new QAction(this);
+    scrollBottomAction->setShortcut(QKeySequence("Ctrl+End"));
+    connect(scrollBottomAction, SIGNAL(triggered()),
+            this, SLOT(scrollToBottom()));
+    this->addAction(scrollBottomAction);
+
+
+
     // Add the default "demo" post
     ASActivity *demoActivity = new ASActivity(demoPostData, this);
-    Post *demoPost = new Post(pController, demoActivity, false, this);
+    Post *demoPost = new Post(demoActivity,
+                              highlightColorsList,
+                              false, // Not highlighted
+                              false, // Not standalone
+                              pController,
+                              this);
     demoPost->setMinMaxHeight(this->minMaxHeightForPosts);
 
     postsInTimeline.append(demoPost);
@@ -259,7 +317,6 @@ void TimeLine::requestTimelinePage()
         this->pController->getFavoritesTimeline(this->timelineOffset);
         break;
     }
-
 }
 
 
@@ -291,6 +348,8 @@ void TimeLine::resizePosts()
     {
         post->setMinMaxHeight(this->minMaxHeightForPosts);
 
+        post->resetResizesCount();
+
         // Force a Post() resizeEvent, which will call
         // setPostContents() and setPostHeight()
         post->resize(post->width() - 1,
@@ -307,6 +366,20 @@ void TimeLine::markPostsAsRead()
 }
 
 
+void TimeLine::updateFuzzyTimestamps()
+{
+    foreach (Post *post, postsInTimeline)
+    {
+        post->setFuzzyTimestamps();
+    }
+}
+
+void TimeLine::updateHighlightColors(QStringList colorList)
+{
+    highlightColorsList = colorList;
+}
+
+
 
 /*********************************************************/
 /*********************************************************/
@@ -326,7 +399,8 @@ void TimeLine::setTimeLineContents(QVariantList postList, int postsPerPage,
     qDebug() << "Removing previous posts from timeline";
     this->clearTimeLineContents();
 
-    emit scrollToTop(); // ask mainWindow to scroll the QScrollArea containing the timeline to the top
+    // Ask mainWindow to scroll the QScrollArea containing the timeline to the top
+    emit scrollTo(QAbstractSlider::SliderToMinimum);
 
     // Hide the widget while it reloads; helps performance a lot
     this->hide();
@@ -338,6 +412,8 @@ void TimeLine::setTimeLineContents(QVariantList postList, int postsPerPage,
     bool allNewPostsCounted = false;
     QString newestPostId; // Here we'll store the postID for the first (newest) post in the timeline
                           // With it, we can know how many new posts (if any) we receive next time
+    newHighlightedPostsCount = 0;
+
 
     // Fill timeline with new contents
     foreach (QVariant singlePost, postList)
@@ -350,7 +426,7 @@ void TimeLine::setTimeLineContents(QVariantList postList, int postsPerPage,
             qDebug() << previousPageLink;
             qDebug() << nextPageLink;
 
-            this->postIsNew = false;
+            bool postIsNew = false;
 
             QVariantMap activityMap;
 
@@ -370,8 +446,12 @@ void TimeLine::setTimeLineContents(QVariantList postList, int postsPerPage,
             }
 
             ASActivity *activity = new ASActivity(activityMap, this);
+
+            // See if it's deleted
             QString postDeletedTime = activity->object()->getDeletedTime();
 
+            // See if we have to filter it out (or highlight it)
+            int filtered = this->fChecker->validateActivity(activity);
 
             if (newestPostId.isEmpty()) // only first time, for newest post
             {
@@ -397,7 +477,9 @@ void TimeLine::setTimeLineContents(QVariantList postList, int postsPerPage,
                 {
                     // If post is NOT deleted, and not ours, add it to the count
                     if (postDeletedTime.isEmpty()
-                        && activity->getAuthorId() != pController->currentUserId())
+                        && filtered != FilterChecker::FilterOut
+                        && activity->author()->getId() != pController->currentUserId()
+                        && activity->object()->author()->getId() != pController->currentUserId())
                     {
                         ++newPostCount;
 
@@ -409,17 +491,33 @@ void TimeLine::setTimeLineContents(QVariantList postList, int postsPerPage,
 
 
 
-            if (postDeletedTime.isEmpty())  // if post was NOT deleted
+            // If post was NOT deleted, and not filtered out
+            if (postDeletedTime.isEmpty() && filtered != FilterChecker::FilterOut)
             {
-                Post *newPost = new Post(pController,
-                                         activity,
+                bool highlightedByFilter = false;
+                if (filtered == FilterChecker::Highlight)
+                {
+                    highlightedByFilter = true;
+                    // FIXME, count highlighted posts
+                }
+
+                Post *newPost = new Post(activity,
+                                         highlightColorsList,
+                                         highlightedByFilter,
                                          false,  // NOT standalone
+                                         pController,
                                          this);
                 if (postIsNew)
                 {
                     newPost->setPostAsNew();
                     connect(newPost, SIGNAL(postRead()),
                             this, SLOT(decreaseUnreadPostsCount()));
+
+                    if (newPost->getHighlightType() != Post::NoHighlight)
+                    {
+                        ++this->newHighlightedPostsCount;
+                    }
+
                 }
                 this->postsInTimeline.append(newPost);
                 this->postsLayout->addWidget(newPost);
@@ -431,8 +529,11 @@ void TimeLine::setTimeLineContents(QVariantList postList, int postsPerPage,
             }
             else
             {
-                // If there's a "deleted" key, ignore this post
-                qDebug() << "This post was deleted on" << postDeletedTime << " / Not adding.";
+                // If there's a "deleted" key, or it's filtered out, ignore this post
+                qDebug() << "This post has been filtered out, or was deleted on"
+                         << postDeletedTime << " / Not adding.";
+                qDebug() << "Filter action:" << filtered
+                         << "(0=filter out; 1=highlight, 999=no filtering)";
 
                 delete activity;
             }
@@ -451,12 +552,16 @@ void TimeLine::setTimeLineContents(QVariantList postList, int postsPerPage,
 
     this->previousNewestPostId = newestPostId;
     this->unreadPostsCount = newPostCount;
-    qDebug() << "New posts:" << newPostCount << "; Newest post ID:" << previousNewestPostId;
+    qDebug() << "New posts:" << newPostCount << "; Newest post ID:" << previousNewestPostId
+             << "\nHighlighted posts:" << newHighlightedPostsCount;
+
 
     this->updateCurrentPageNumber();
     this->resizePosts();
 
-    emit timelineRendered(this->timelineType, unreadPostsCount);
+    emit timelineRendered(this->timelineType,
+                          unreadPostsCount,
+                          newHighlightedPostsCount);
 
     // Show timeline again, since everything is added and drawn
     this->show();
@@ -538,8 +643,10 @@ void TimeLine::goToFirstPage()
     qDebug() << "TimeLine::goToFirstPage()";
 
     this->timelineOffset = 0;
-
     this->requestTimelinePage();
+
+    /// new method!
+    //pController->getTimeline(this->timelineType, this->previousPageLink);
 }
 
 
@@ -570,6 +677,38 @@ void TimeLine::goToNextPage()
 
 
 
+void TimeLine::scrollUp()
+{
+    emit scrollTo(QAbstractSlider::SliderSingleStepSub);
+}
+
+void TimeLine::scrollDown()
+{
+    emit scrollTo(QAbstractSlider::SliderSingleStepAdd);
+}
+
+void TimeLine::scrollPageUp()
+{
+    emit scrollTo(QAbstractSlider::SliderPageStepSub);
+}
+
+void TimeLine::scrollPageDown()
+{
+    emit scrollTo(QAbstractSlider::SliderPageStepAdd);
+}
+
+void TimeLine::scrollToTop()
+{
+    emit scrollTo(QAbstractSlider::SliderToMinimum);
+}
+
+void TimeLine::scrollToBottom()
+{
+    emit scrollTo(QAbstractSlider::SliderToMaximum);
+}
+
+
+
 /*
  * Decrease internal counter of unread posts (by 1), and inform
  * the parent window, so it can update its tab titles
@@ -579,5 +718,7 @@ void TimeLine::decreaseUnreadPostsCount()
 {
     --unreadPostsCount;
 
-    emit unreadPostsCountChanged(this->timelineType, this->unreadPostsCount);
+    emit unreadPostsCountChanged(this->timelineType,
+                                 this->unreadPostsCount,
+                                 this->newHighlightedPostsCount);
 }
